unit AliasDialog;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons;

type
  TheMode = (Open,Save,Alias,Delete);

  TAliasForm = class(TForm)
    cbAlias: TComboBox;
    lblAlias: TLabel;
    btnNew: TSpeedButton;
    lbTables: TListBox;
    edFile: TEdit;
    lblFile: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    procedure btnCancelClick(Sender: TObject);
    procedure cbAliasChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lbTablesClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    TheMode: TheMode;
    SaveItem: Integer;
    procedure SetMode(aMode: TheMode);
  public
    { Public declarations }
    property DlgMode: TheMode write SetMode;
  end;

var
  AliasForm: TAliasForm;

implementation

{$R *.DFM}

uses dbtables, NewAliasDialog;

procedure TAliasForm.SetMode(aMode: TheMode);
begin
  TheMode := aMode;
  case aMode of
    Open: begin
      Caption := 'Open Table';
      lblAlias.Caption := 'Look &in:';
      btnOk.Caption := '&Open';
      edFile.Visible := True;
      lblFile.Visible := True;
      edFile.ReadOnly := True;
      btnNew.Visible := True;
      HelpContext := 280;
      end;
    Save: begin
      Caption := 'Save New Table';
      lblAlias.Caption := 'Save &in:';
      btnOk.Caption := '&Save';
      edFile.Visible := True;
      lblFile.Visible := True;
      edFile.ReadOnly := False;
      btnNew.Visible := True;
      HelpContext := 281;
      end;
    Alias: begin
      Caption := 'Open Database';
      lblAlias.Caption := 'Open &tables in:';
      btnOk.Caption := '&Open';
      edFile.Visible := False;
      lblFile.Visible := False;
      edFile.ReadOnly := True;
      btnNew.Visible := True;
      HelpContext := 283;
      end;
    Delete: begin
      Caption := 'Delete Alias';
      lblAlias.Caption := 'Delete &alias:';
      btnOk.Caption := '&Delete';
      edFile.Visible := False;
      lblFile.Visible := False;
      edFile.ReadOnly := True;
      btnNew.Visible := False;
      HelpContext := 284;
      end;
  end {case};
  Session.GetAliasNames(cbAlias.Items);
  lbTables.Items.Clear;
  cbAlias.ItemIndex := 0;
  try
    Session.GetTableNames(cbAlias.Text,'*.dbf',True,False,lbTables.Items);
  except
    
  end;
  edFile.Text := '';
end;

procedure TAliasForm.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TAliasForm.cbAliasChange(Sender: TObject);
var
  Cntr: Integer;
  iTemp: Integer;
  MaxText: Integer;
begin
  Session.GetTableNames(cbAlias.Text,'*.dbf',True,False,lbTables.Items);
  MaxText := 0;
  for Cntr := 0 to lbTables.Items.Count - 1 do begin
    iTemp := Canvas.TextWidth(lbTables.Items[Cntr]);
    if iTemp > MaxText then
      MaxText := iTemp;
    {fi}
    end;
  {od for}
  if MaxText = 0 then
    lbTables.Columns := 0
  else
    lbTables.Columns := lbTables.Width div (MaxText+5);
  {fi}
  lbTables.SetFocus;
end;

procedure TAliasForm.FormCreate(Sender: TObject);
begin
  SaveItem := 0;
  Session.ConfigMode := cmPersistent;
end;

procedure TAliasForm.lbTablesClick(Sender: TObject);
begin
  edFile.Text := lbTables.Items[lbTables.ItemIndex];
end;

procedure TAliasForm.btnOKClick(Sender: TObject);
begin
  SaveItem := cbAlias.ItemIndex;
  case TheMode of
    Alias: begin;
      if lbTables.Items.Count > 0 then
        ModalResult := mrOk
      else
        ShowMessage('There are no tables in ' + cbAlias.Text)
      {fi}
      end;
    Open, Save: begin;
      if (Length(edFile.Text) > 0) and (Length(cbAlias.Text) > 0) then begin
        edFile.Text := ChangeFileExt(edFile.Text,'.dbf');
        ModalResult := mrOk;
        end
      else
        ShowMessage('An alias and a filename must be specified.');
      {fi}
      end;
    Delete:
      if MessageDlg('Are you sure that you want to delete '+cbAlias.Text+'?',
           mtConfirmation,[mbYes,mbNo],0) = mrYes then
        ModalResult := mrOk
      else
        ModalResult := mrCancel;
      {fi}
  end {case};
end;

procedure TAliasForm.btnNewClick(Sender: TObject);
begin
  with FormNewAlias do begin
    DriveComboBox1.Drive := 'C';
    if FormNewAlias.ShowModal = mrOk then
      try
        Session.AddStandardAlias(edAlias.Text,
           DirectoryListBox1.GetItemPath(DirectoryListBox1.ItemIndex),'DBASE');
        Session.SaveConfigFile;
        cbAlias.Items.Add(edAlias.Text);
      except
        on E: Exception do
          ShowMessage('Error creating alias "'+edAlias.Text+'"');
      end {try};
      end;
    {fi}
  {od with}
end;

procedure TAliasForm.FormActivate(Sender: TObject);
begin
  cbAlias.ItemIndex := SaveItem;
  cbAliasChange(Self);
end;

end.
