{***********************************************************************}
{                                                                       }
{  Childwin defines the form used to display a dBASE table in table or  }
{  list form.  The form includes a FieldStructure array documents the   }
{  field structure contained in the file header.  It is used to display }
{  and print the field structure.                                       }
{                                                                       }
{  Copyright (c) 2000, 2003 Douglas P. Baird - dpbaird@sprynet.com      }
{                                                                       }
{***********************************************************************}

unit Childwin;

interface

uses Windows, Messages, Classes, Graphics, Forms, Controls, StdCtrls, Grids,
  DBGrids, Db, DBTables, DBCtrls, dbvConst, Menus;
const
  dHeight = 129;
  dWidth = 265;
type
  FieldStructure = record  { Description of field in table }
    Name: string[32];
    FieldType: Char;
    Length: string[3];
    Decimal: string[2];
    Required: Char;
  end;
  TMDIChild = class(TForm)
    DataSource1: TDataSource;
    Table1: TTable;
    DBGrid1: TDBGrid;
    StringGrid1: TStringGrid;
    DBMemo1: TDBMemo;
    PopupMenu1: TPopupMenu;
    Restore1: TMenuItem;     { Used to control the size of DBMemo1 }
    Maximize1: TMenuItem;
    Larger1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure DBGrid1ColEnter(Sender: TObject);
    procedure DBMemo1Exit(Sender: TObject);
    procedure StringGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure DBGrid1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Restore1Click(Sender: TObject);
    procedure Maximize1Click(Sender: TObject);
    procedure Larger1Click(Sender: TObject);
    procedure DBGrid1CellClick(Column: TColumn);
    procedure Table1AfterScroll(DataSet: TDataSet);
  private
    { Private declarations }
    SkipResize: Boolean;
    function OpenDbFile: Boolean;
    procedure GetStructure(var msg: TMessage); message GetStruct;
    function GetFilePath: string;
    function GetTablePath: string;
    procedure ShowMemo(aCol,aRow: Integer);
  public
    { Public declarations }
    FieldCnt: Integer;
    BrowseMode: Boolean;
    EditMode: Boolean;
    ShowDeleted: Boolean;
    MemoBlock: Word;
    dBASEtype: DbType;
    TableStructure: array of FieldStructure;
    CallName: string;
    procedure ListFields;
    property TablePath: string read GetTablePath;
    property FilePath: string read GetFilePath;
    procedure SetCaption;
  end;  { TMDIChild }

implementation

uses Math, SysUtils, Dialogs, Main;

{$R *.DFM}

procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Table1.Active := False;
  Table1.Free;
  SetLength(TableStructure,0);
  if MainForm.MDIChildCount = 1 then
    MainForm.SetDbActions(False,False);
  {fi}
  Action := caFree;
end;

procedure TMDIChild.FormResize(Sender: TObject);
var
  aHeight, aWidth: Integer;
begin
  if not SkipResize then begin
    DbGrid1.Height := ClientHeight;
    DbGrid1.Width := ClientWidth;
    StringGrid1.Height := ClientHeight;
    StringGrid1.Width := ClientWidth;
    StringGrid1.ColWidths[1] := StringGrid1.Width - StringGrid1.ColWidths[0];  
    aHeight := Min(dHeight,ClientHeight);
    aWidth := Min(dWidth,ClientWidth);
    DBMemo1.Height := aHeight;
    DBMemo1.Width := aWidth;
    DBMemo1.Left := (ClientWidth - aWidth) div 2;
    DBMemo1.Top := (ClientHeight - aHeight) div 2;
    SetCaption;
    end;
  {fi skip resize}
end;

procedure TMDIChild.FormActivate(Sender: TObject);
begin
  SkipResize := False;
  with MainForm do begin
    SetDbActions(True,EditMode);
    SetNav;
    if ShowDeleted then begin
      mToggleDeleted.Caption := 'Hide Deleted';
      if RecIsDeleted(Table1) then begin
        StatusBar.Panels[1].Text := 'Record is deleted';
        MainForm.RecordUndelete.Enabled := True;
        end
      else begin
        StatusBar.Panels[1].Text := '';
        MainForm.RecordUndelete.Enabled := False;
        end
      {fi}
      end
    else begin
      mToggleDeleted.Caption := 'Show Deleted';
      StatusBar.Panels[1].Text := '';
      end;
    {fi}
    DbBrowse.Checked := BrowseMode;
    DbList.Checked := not BrowseMode;
    FileEdit1.Checked := EditMode
    end;
  {od with}
end;

{***********************************************************************}
{  OpenDbFile initializes Table1 and opens the table.  It utilizes the  }
{  following public variables in MainForm:                              }
{    aFileName - specifies the name of the file to be opened            }
{    aFileType - specifies the Xbase type encoded in the file header    }
{  If the table is opened successfully, then DBGrid1 is refreshed and   }
{  the size of StringGrid1 is set.                                      }
{                                                                       }
{  Called by:                                                           }
{    FormCreate                                                         }
{                                                                       }
{  MainForm performs a number of tests on the validity of the header    }
{  of the file before attempting to create an MDIChild.  It is possible }
{  that the DBE will signal exceptions because of other faults in the   }
{  file format.  During development, some of the header error           }
{  conditions also created exceptions when exiting the program.  The    }
{  The tests in MainForm eliminated all such exceptions that were       }
{  detected during the development.  It is possible that other problems }
{  with malformed files will be detected during the execution of        }
{  OpenDbFile.                                                          }
{***********************************************************************}

function TMDIChild.OpenDbFile: Boolean;
var
  aName: string;
{func/proc}
  procedure Cleanup;
  begin
    if Table1.Active then
      try
        Table1.Active := False;
      except
        { Kill error message }
      end;
    {fi}
  end;
begin  { TMDIChild.OpenDbFile }
  aName := MainForm.aFileName;
  Table1.DatabaseName := ExtractFileDir(aName);
  Table1.TableName := ExtractFileName(aName);
  if MainForm.aFileType = FoxProType then
    Table1.TableType := ttFoxPro
  else
    Table1.TableType := ttDBase;
  {fi}
  Result := True;
  try
    Table1.Active := True;
  except
    on E: EDBEngineError do begin
      MessageDlg('Error opening table '+Table1.DatabaseName+'\'+
      Table1.TableName+#13#13+E.Message+#13#13'Press F1 for More Information',
      mtError,[mbOk],55);
      Cleanup;
      Result := False;
      end;
    on E: EAccessViolation do begin
      MessageDlg('Database Engine error. '+E.Message+#13#13+
          'May generate error messages on program exit.',mtError,[mbOk],0);
      Cleanup;
      Result := False;
      end;
    on E: EInvalidPointer do begin
      MessageDlg('Database Engine error. '+E.Message+#13#13+
          'May generate error messages on program exit.',mtError,[mbOk],0);
      Cleanup;
      Result := False;
      end;
  else begin
      MessageDlg('Error opening table. '#13#13+
        'May generate error messages on program exit.',
        mtError,[mbOk],0);
      Result := False;
      Cleanup;
      end;
    {esle}
  end; {try}
  if Result then begin
    DBGrid1.Refresh;
    MainForm.SetDbActions(True,EditMode);
    CallName := MainForm.GetCallName(ChangeFileExt(Table1.TableName,''));
    SetCaption;
    with Table1.FieldList do begin
      FieldCnt := Count;
      StringGrid1.RowCount := FieldCnt;
      end;
    {od with}
    end;
  {fi}
end;  { TMDIChild.OpenDbFile }

procedure TMDIChild.DBGrid1ColEnter(Sender: TObject);
begin
  with DbMemo1 do begin
    DataField := '';
    if (DbGrid1.SelectedField.DataType = ftMemo) then begin
      DataField := DbGrid1.SelectedField.FieldName;
      Visible := EditMode or (DbGrid1.SelectedField.Text = '(MEMO)');
      ReadOnly := not EditMode;
      if Visible then
        SetFocus;
      {fi}
      end
    else
      Visible := False;
    {fi}
    end;
  {od with}
end;

procedure TMDIChild.DBGrid1CellClick(Column: TColumn);
begin
  DBGrid1ColEnter(Self);
end;

procedure TMDIChild.DBMemo1Exit(Sender: TObject);
begin
  DbMemo1.Visible := False;
  if DbGrid1.Visible then
    DbGrid1.SetFocus
  else if StringGrid1.Visible then
    StringGrid1.SetFocus;
  {fi}
end;

{***********************************************************************}
{                                                                       }
{  ListFields fills in StringGrid1 with the contents, including field   }
{  names of the current record.                                         }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.ListFields;
var
  Cntr: Integer;
  Temp: string;
begin
  if ShowDeleted and RecIsDeleted(Table1) then begin
    MainForm.StatusBar.Panels[1].Text := 'Record is deleted';
    MainForm.RecordUndelete.Enabled := True;
    end
  else begin
    MainForm.StatusBar.Panels[1].Text := '';
    MainForm.RecordUndelete.Enabled := False;
    end;
  {fi}
  for Cntr := 0 to FieldCnt-1 do begin
    StringGrid1.Cells[0,Cntr] := Table1.Fields[Cntr].FieldName;
    if Table1.Fields[Cntr].IsBlob then begin
      with Table1.FieldDefs.Items[Cntr] do
        if DataType = ftTypedBinary then
          Temp := '[Typed Binary]'
        else if DataType = ftBlob then
          Temp := '[Blob]'
        else
          Temp := '[Memo]';
      {od with}
      if Table1.Fields.Fields[Cntr].AsString > ' '  then
        StringGrid1.Cells[1,Cntr] := UpperCase(Temp)
      else
        StringGrid1.Cells[1,Cntr] := Temp;
      end
    else
      StringGrid1.Cells[1,Cntr] := Table1.Fields[Cntr].AsString;
    {fi}
    end;
  {od for}
end;  { TMDIChild.ListFields }

{***********************************************************************}
{                                                                       }
{  ShowMemo makes the control DbMemo1 visible and displays the contents }
{  of the memo field that has just been entered.                        }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.ShowMemo(aCol,aRow: Integer);
begin
  if (StringGrid1.Cells[aCol,aRow] = '[MEMO]') then begin
    DbMemo1.DataField := StringGrid1.Cells[0,aRow];
    DbMemo1.Visible := True;
    end
  else
    DbMemo1.Visible := False;
  {fi}
end;

procedure TMDIChild.StringGrid1SelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  ShowMemo(aCol,aRow);
end;

{***********************************************************************}
{                                                                       }
{  FormCreate is the OnCreate procedure called when a new MDIChild is   }
{  created.  It uses the following public variable in MainForm:         }
{    sFileType - which establishes the TableType                        }
{
{  FormCreate uses a message to call GetStructure that creates the      }
{  TableStructure array.                                                }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.FormCreate(Sender: TObject);
{func/proc}
begin  { TMDIChild.FormCreate }
  BrowseMode := True;
  EditMode := False;
  FormResize(Sender);
  if OpenDbFile then begin
    MainForm.MDICreated := True;
    PostMessage(Self.Handle,GetStruct,Ord(MainForm.aFileType),0);
    end
  else begin
    MainForm.MDICreated := False;
    if MainForm.MDIChildCount = 1 then begin
      MainForm.SetDbActions(False,False);
      end;
    {fi}
    end;
  {fi}
end;  { TMDIChild.FormCreate }

{***********************************************************************}
{                                                                       }
{  GetStructure reads the file header of a XBase table and creates the  }
{  TableStructure array.  It must be executed asyncronously with        }
{  FormCreate to allow table creation                                   }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.GetStructure(var msg: TMessage);
var
  F: file of byte;
  aBuffer: array[0..47] of byte;
  DescrSize: Integer;
  NameSize: Integer;
  TypeOffset: Integer;
  LengthOffset: Integer;
  DecimalOffset: Integer;
  DescrNmbr: Integer;
  DescrStart: Integer;
  Cntr: Integer;
  sTemp: string;
  {func/proc}
  procedure MoveName(aDescr: Integer);
  var
    Cntr: Integer;
    aChar: Char;
  begin  {MoveName}
    TableStructure[aDescr].Name[0] := Char(32);
    for Cntr := 0 to NameSize - 1 do begin
      if aBuffer[Cntr] = 0 then
        aChar := ' '
      else
        aChar := Char(aBuffer[Cntr]);
      {fi}
      TableStructure[aDescr].Name[Cntr+1] := aChar;
      end;
    {od for}
    for Cntr := NameSize+1 to 32 do
      TableStructure[aDescr].Name[Cntr] := ' ';
    {od for}
  end;  { MoveName}
begin  { TMDIChild.GetStructure }
  Table1.Active := False;
  if Table1.DatabaseName[2] <> ':' then
    sTemp := MainForm.AliasToFile(Table1.DatabaseName+'\'+Table1.TableName)
  else begin
    sTemp := Table1.DatabaseName;
    if Length(sTemp) > 3 then
      sTemp := sTemp + '\';
    {fi}
    sTemp := sTemp + Table1.TableName;
    end;
  {fi}
  if FileExists(sTemp) then begin
    AssignFile(F,sTemp);
    Reset(F);
    BlockRead(F,aBuffer,48);
    dBASEtype := DbType(msg.WParam);
    if dBASEtype = Lvl7Type then begin
      DescrSize := 48;
      NameSize := 32;
      TypeOffset := 32;
      LengthOffset := 33;
      DecimalOffset := 34;
      DescrStart := 68;
      end
    else begin
      DescrSize := 32;
      NameSize := 11;
      TypeOffset := 11;
      LengthOffset := 16;
      DecimalOffset := 17;
      DescrStart := 32;
      end;
    {fi}
    DescrNmbr := ((256*aBuffer[9] + aBuffer[8]) - DescrStart - 1) div DescrSize;
    SetLength(TableStructure,DescrNmbr);
    Seek(F,DescrStart);
    for Cntr := 0 to DescrNmbr-1 do begin
      BlockRead(F,aBuffer,DescrSize);
      MoveName(Cntr);
      if TableStructure[Cntr].Name[1] = Char(13) then begin
        DescrNmbr := Cntr;
        SetLength(TableStructure,DescrNmbr);
        Break;
        end;
      {fi}
      TableStructure[Cntr].FieldType := Char(aBuffer[TypeOffset]);
      TableStructure[Cntr].Length := IntToStr(aBuffer[LengthOffset]);
      if Pos(TableStructure[Cntr].FieldType,'NF') = 0 then
        TableStructure[Cntr].Decimal := ''
      else
        TableStructure[Cntr].Decimal := IntToStr(aBuffer[DecimalOffset]);
      end;
    {od for}
    CloseFile(F);
    MemoBlock := 0;
    if dBASEtype = FoxProType then
      sTemp := ChangeFileExt(sTemp,'.FPT')
    else
      sTemp := ChangeFileExt(sTemp,'.DBT');
    {fi}
    if FileExists(sTemp) then begin
      AssignFile(F,sTemp);
      Reset(F);
      BlockRead(F,aBuffer,24);
      if dBASEtype = FoxProType then
        MemoBlock := 256*aBuffer[6] + aBuffer[7]
      else
        MemoBlock := 256*aBuffer[21] + aBuffer[20];
      {fii}
      CloseFile(F);
      end;
    {fi}
    try
      Table1.Active := True;
      for Cntr := 0 to Table1.FieldCount-1 do begin
        if (dBASEtype = Lvl7Type) and
           Table1.Fields.Fields[Cntr].Required then
          TableStructure[Cntr].Required := 'Y'
        else
          TableStructure[Cntr].Required := 'N';
        {fi}
        end;
      {od for}
    except
      { No error message needed }
    end;
    end;
  {fi}
end;  { TMDIChild.GetStructure }

function TMDIChild.GetTablePath: string;
begin  { TMDIChild.GetTablePath }
  with Table1 do
    if (Length(DatabaseName) = 3) and (DatabaseName[2] = ':') then
      Result := DatabaseName + TableName
    else
     Result := DatabaseName + '\'+ TableName;
    {fi}
  {od with}
end;  { TMDIChild.GetTablePath }

function TMDIChild.GetFilePath: string;
begin  { TMDIChild.GetFilePath }
  with Table1 do
    if DatabaseName[2] = ':' then begin { not an alias }
      if Length(DatabaseName) = 3 then
        Result := DatabaseName+TableName
      else
        Result := DatabaseName+'\'+ TableName;
      {fi}
      end
    else
      Result := MainForm.AliasToFile(DatabaseName+'\'+TableName);
    {fi}
  {od with}
end;  { TMDIChild.GetFilePath }

procedure TMDIChild.DBGrid1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_PRIOR) or (Key = VK_NEXT) or (Key = VK_UP) or
     (Key = VK_DOWN) or (Key = VK_TAB) or (Key = VK_HOME) or
     (Key = VK_END) then
    MainForm.SetNav;
  {fi}
  if (DbGrid1.SelectedField.DataType = ftMemo) and
     ((Key = VK_UP) or (Key = VK_Down)) then
    DBGrid1ColEnter(Sender);
  {fi}
end;

procedure TMDIChild.Restore1Click(Sender: TObject);
var
  aHeight, aWidth: Integer;
begin
  SkipResize := True;
  aHeight := Min(dHeight,ClientHeight);
  aWidth := Min(dWidth,ClientWidth);
  DbMemo1.Height := aHeight;
  DbMemo1.Width := aWidth;
  DbMemo1.Left := (ClientWidth - aWidth) div 2;
  DbMemo1.Top := (ClientHeight - aHeight) div 2;
  SkipResize := False;
end;  { TMDIChild.Restore1Click }

procedure TMDIChild.Maximize1Click(Sender: TObject);
var
  aHeight, aWidth: Integer;
begin
  SkipResize := True;
  aHeight := ClientHeight;
  aWidth := ClientWidth;
  DbMemo1.Height := aHeight;
  DbMemo1.Width :=aWidth;
  DbMemo1.Left := 0;
  DbMemo1.Top := 0;
  SkipResize := False;
end;  { TMDIChild.Maximize1Click }

procedure TMDIChild.Larger1Click(Sender: TObject);
var
  aHeight, aWidth: Integer;
begin
  SkipResize := True;
  aHeight := Min(DbMemo1.Height+(DbMemo1.Height div 2),ClientHeight);
  aWidth := Min(DbMemo1.Width+(DbMemo1.Width div 2),ClientWidth);
  DbMemo1.Height := aHeight;
  DbMemo1.Width := aWidth;
  DbMemo1.Left := (ClientWidth - aWidth) div 2;
  DbMemo1.Top := (ClientHeight - aHeight) div 2;
  SkipResize := False;
end;  { TMDIChild.Larger1Click }

procedure TMDIChild.Table1AfterScroll(DataSet: TDataSet);
begin
  if DBGrid1.Visible then begin
    if ShowDeleted then
      if RecIsDeleted(Table1) then begin
        MainForm.StatusBar.Panels[1].Text := 'Record is deleted';
        MainForm.RecordUndelete.Enabled := True;
        end
      else begin
        MainForm.StatusBar.Panels[1].Text := '';
        MainForm.RecordUndelete.Enabled := False;
        end;
      {fi}
    {fi}
    DBGrid1ColEnter(Self);
    {fi}
    end;
  {fi}
end;  { TMDIChild.Table1AfterScroll }

procedure TMDIChild.SetCaption;
const
  Overhead = 120;
var
  sTemp: string;
  aWidth: Integer;
  Cntr: Integer;
begin
    sTemp := TablePath;
    with Canvas do
      if TextWidth(CallName+'-'+sTemp) > ClientWidth - Overhead then begin
        aWidth := TextWidth(CallName);
        if aWidth > ClientWidth - Overhead then
          Caption := CallName
        else begin
          for Cntr := 1 to Length(sTemp) do
            if TextWidth(Copy(sTemp,1,Cntr)) + aWidth >
                   ClientWidth - Overhead then begin
              Caption := CallName + '-' + Copy(sTemp,1,Cntr-1) + '...';
              Break;
              end;
            {fi}
          end;
        {fi}
        end
      else
        Caption := CallName + '-' + sTemp;
      {fi}
    {od with}
end;  { TMDIChild.SetCaption }

end.

