unit CopyForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, DataGrid, ChildWin, Buttons, dbtables, ComCtrls,
  ExtCtrls;

type
  TFormCopy = class(TForm)
    InDataGrid: TDataGrid;
    Label1: TLabel;
    OutDataGrid: TDataGrid;
    Label2: TLabel;
    btnCopy: TBitBtn;
    btnAbort: TBitBtn;
    btnClear: TBitBtn;
    btnCancel: TBitBtn;
    ProgressBar1: TProgressBar;
    Image1: TImage;
    procedure btnAbortClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure InDataGridMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OutDataGridDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure OutDataGridDragDrop(Sender, Source: TObject; X, Y: Integer);
  private
    { Private declarations }
    InTbl, OutTbl: TTable;
    Running: Boolean;
    DragField: String;
    procedure SetButtons(SetAs: Boolean);
  public
    { Public declarations }
    procedure CopyLists(InChild, OutChild: TMDIChild);
  end;  { TFormCopy }

var
  FormCopy: TFormCopy;

implementation

{$R *.DFM}

uses dbvConst, DB;

procedure TFormCopy.CopyLists(InChild, OutChild: TMDIChild);
var
  Cntr: Integer;
  InRow: Integer;
  IsAutoInc: Boolean;
{func/proc}
  function FindField(aName: string): Integer;
  var
    Cntr: Integer;
    InName: string;
  begin
    Result := -1;
    for Cntr := 1 to InDataGrid.RowCount+1 do begin
        if OutChild.dBASEtype = Lvl7Type then
        InName := Trim(InDataGrid.Cells[0,Cntr])
      else
        InName := UpperCase(Trim(Copy(InDataGrid.Cells[0,Cntr],0,10)));
      {fi}
      if Trim(aName) = InName then begin
        Result := Cntr;
        Break;
        end;
      {fi}
      end;
    {od for}
  end;
begin  { TFormCopy.CopyLists }
  IsAutoInc := False;
  OutDataGrid.Columns[0].PickList.Clear;
  OutDataGrid.Columns[0].PickList.Add('');
  with InChild do begin
    InDataGrid.RowCount := High(TableStructure)+2;
    for Cntr := 0 to High(TableStructure) do begin
      OutDataGrid.Columns[0].PickList.Add(Trim(TableStructure[Cntr].Name));
      InDataGrid.Cells[0,Cntr+1] := Trim(TableStructure[Cntr].Name);
      InDataGrid.Cells[1,Cntr+1] := TableStructure[Cntr].FieldType;
      InDataGrid.Cells[2,Cntr+1] := TableStructure[Cntr].Length;
      InDataGrid.Cells[3,Cntr+1] := TableStructure[Cntr].Decimal;
      if InDataGrid.Cells[1,Cntr+1] = '+' then
        IsAutoInc := True;
      {fi}
      end;
    {od for}
    end;
  {od with}
  with OutChild do begin
    OutDataGrid.RowCount := High(TableStructure)+2;
    for Cntr := 0 to High(TableStructure) do begin
      OutDataGrid.Cells[1,Cntr+1] := TableStructure[Cntr].Name;
      OutDataGrid.Cells[2,Cntr+1] := TableStructure[Cntr].FieldType;
      OutDataGrid.Cells[3,Cntr+1] := TableStructure[Cntr].Length;
      OutDataGrid.Cells[4,Cntr+1] := TableStructure[Cntr].Decimal;
      {fi}
      end;
    {od for}
    end;
  {od with}
  for Cntr := 1 to OutDataGrid.RowCount-1 do begin
    InRow := FindField(OutDataGrid.Cells[1,Cntr]);
    if InRow > 0 then
      OutDataGrid.Cells[0,Cntr] := InDataGrid.Cells[0,InRow];
    end;
  {od for}
  if IsAutoInc then
    MessageDlg('The table '+InChild.Table1.TableName+
         ' contains an AutoIncrement field.'#13#13+
         'There is a potential for the loss of database integrity',
         mtWarning,[mbOk],0);
  {fi}
  InTbl := InChild.Table1;
  OutTbl := OutChild.Table1;
  SetButtons(True);
end;  { TFormCopy.CopyLists }

procedure TFormCopy.btnAbortClick(Sender: TObject);
begin
  Running := (MessageDlg('Do you really want to stop scanning the directory?',
                 mtConfirmation,[mbYes,mbNo],0) = mrNo);
  if not Running then begin
    OutDataGrid.Columns[0].PickList.Clear;
    InDataGrid.RowCount := 1;
    OutDataGrid.RowCount := 1;
    ModalResult := mrCancel;
    end;
  {fi}
end;

procedure TFormCopy.btnCopyClick(Sender: TObject);
var
  Cntr: Integer;
  InField: string;
  OutField: string;
{func/proc}
  function ShowError: string;
  begin
    Result := 'Conversion between input field "'+InField+
      '" and output field "'+OutField+'" not supported.';
  end;
begin
  SetButtons(False);
  Running := True;
  InTbl.First;
  OutTbl.Active := False;
  OutTbl.ReadOnly := False;
  OutTbl.Active := True;
  ProgressBar1.Max := InTbl.RecordCount;
  OutTbl.DisableControls;
  while not InTbl.Eof do begin
    OutTbl.Append;
    try
      for Cntr := 1 to OutDataGrid.RowCount-1 do begin
        InField := Trim(OutDataGrid.Cells[0,Cntr]);
        OutField := Trim(OutDataGrid.Cells[1,Cntr]);
        if (Length(InField) = 0) or InTbl.FieldByName(InField).IsNull then
          Continue;
        {fi}
        case OutTbl.FieldByName(OutField).DataType of
          ftAutoInc: Continue;
          ftMemo:
            case InTbl.FieldByName(InField).DataType of
              ftMemo: TMemoField(OutTbl.FieldByName(OutField)).Value :=
                      TMemoField(InTbl.FieldByName(InField)).Value;
              ftTypedBinary: raise Exception.Create(ShowError);
              ftDBaseOle: raise Exception.Create(ShowError);
            else  TMemoField(OutTbl.FieldByName(OutField)).Value :=
                  TMemoField(InTbl.FieldByName(InField)).AsString;
            end; {ftMemo}
          ftTypedBinary:
            if InTbl.FieldByName(InField).DataType = ftTypedBinary then
              TMemoField(OutTbl.FieldByName(OutField)).Value :=
              TMemoField(InTbl.FieldByName(InField)).Value
            else
              raise Exception.Create(ShowError);
            {fi}
          ftDBaseOle:
            if InTbl.FieldByName(InField).DataType = ftDBaseOle then
              TMemoField(OutTbl.FieldByName(OutField)).Value :=
              TMemoField(InTbl.FieldByName(InField)).Value
            else
              raise Exception.Create(ShowError);
            {fi}
        else
          case InTbl.FieldByName(InField).DataType of
            ftMemo: OutTbl.FieldValues[OutField] :=
                    TMemoField(InTbl.FieldByName(InField)).Value;
            ftTypedBinary: raise Exception.Create(ShowError);
            ftDBaseOle: raise Exception.Create(ShowError);
          else
            OutTbl.FieldValues[OutField] := InTbl.FieldValues[InField];
          end;
        end; {case}
        end;
      {od for}
      OutTbl.Post;
    except
      on E: Exception do begin
        ShowMessage('Error: '+E.Message);
        Break;
        end;
      {od on}
    end;
    ProgressBar1.StepIt;
    Application.ProcessMessages;
    if Running then
      InTbl.Next
    else
      Break;
    {fi}
    end;
  {od while}
  OutTbl.Active := False;
  OutTbl.ReadOnly := True;
  OutTbl.EnableControls;
  OutTbl.Active := True;
  InDataGrid.RowCount := 1;
  OutDataGrid.RowCount := 1;
  ModalResult := mrOk;
end;

procedure TFormCopy.btnClearClick(Sender: TObject);
begin
  OutDataGrid.Cells[0,OutDataGrid.Row] := '';
end;

procedure TFormCopy.btnCancelClick(Sender: TObject);
begin
  OutDataGrid.Columns[0].PickList.Clear;
  ModalResult := mrCancel;
end;

procedure TFormCopy.SetButtons(SetAs: Boolean);
begin
  btnCopy.Visible := SetAs;
  btnAbort.Visible := not SetAs;
  btnCancel.Visible := SetAs;
  ProgressBar1.Visible := not SetAs;
end;  { TFormCopy.SetButtons }

procedure TFormCopy.InDataGridMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  CellCoord: TGridCoord;
begin
  if Button = mbLeft then
    with Sender as TDataGrid do begin
      CellCoord := MouseCoord(X,Y);
      if CellCoord.Y > 0 then begin
        BeginDrag(False);
        end;
      {fi}
      end;
    {od with}
  {fi}
end;

procedure TFormCopy.OutDataGridDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  CellCoord: TGridCoord;
begin
  with Sender as TDataGrid do begin
    CellCoord := MouseCoord(X,Y);
    if CellCoord.Y > 0 then begin
      Col := CellCoord.X;
      Row := CellCoord.Y;
      SetFocus;
      with Source as TDataGrid do
        DragField := Cells[0,Row];
      {od with}
      Accept := True;
      end;
    {fi}
    end;
  {od with}
end;

procedure TFormCopy.OutDataGridDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  with OutDataGrid do
      Cells[0,Row] := DragField;
  {od with}
end;

end.
