unit DataModForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DataGrid, StdCtrls, Buttons, ChildWin, PrintPrev;

const
  Asterisks =  '{**********************************************'+
               '*************************}';
  Spacer = '{                                                  '+
           '                     }';

type
  TformDataMod = class(TForm)
    dgTables: TDataGrid;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    cbProperties: TCheckBox;
    cbActivate: TCheckBox;
    cbOpen: TCheckBox;
    cbCreate: TCheckBox;
    edModName: TEdit;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    cbPrint: TCheckBox;
    cbSave: TCheckBox;
    Prnt: TPrintPreview;
    procedure btnOKClick(Sender: TObject);
    procedure cbCreateClick(Sender: TObject);
  private
    { Private declarations }
    SaveOpen: Boolean;
    NeedPatch: Boolean;
    procedure SaveTbl(aChild: TMDIChild);
    function ScanMemo: Boolean;
    procedure SavePatch;
  public
    { Public declarations }
    procedure LoadTables;
    procedure SaveCode;
  end;  { TformDataMod }

  TListObj = class(TObject)
    Line: Integer;
    Child: TMDIChild;
    constructor Create(aLine: Integer; aChild: TMDIChild);
    end;

var
  formDataMod: TformDataMod;
  F: TextFile;

implementation

{$R *.DFM}

uses Math, db, Main, dbvConst;

{***********************************************************************}
{                                                                       }
{  LoadTables is called by TMainForm.mDataModClick to initialized this  }
{  form before executing the ShowModal method.                          }
{                                                                       }
{***********************************************************************}

procedure TformDataMod.LoadTables;
var
  Cntr: Integer;
  NameList: TStringList;
  DupsFound: Integer;
  sDupsFound: Integer;
  sTemp: string;
begin { TformDataMod.LoadTables }
  formDataMod.dgTables.RowCount := MainForm.MDIChildCount + 1;
  for Cntr := 1 to MainForm.MDIChildCount do
    with MainForm.MDIChildren[Mainform.MDIChildCount - Cntr] as TMDIChild do
      with formDataMod.dgTables do begin
        sTemp := StringReplace(Copy(CallName,1,62),' ','_',[rfReplaceAll]);
        Cells[0,Cntr] := UpperCase(sTemp[1]) + Copy(sTemp,2,62);
        Cells[1,Cntr] := 's'+Cells[0,Cntr];
        Cells[2,Cntr] := CallName + '-' + TablePath;
        end;
      {od with dgTables}
    {od with MDIChildren}
  {od for}
  NameList := TStringList.Create;
  NameList.Capacity := 2 * MainForm.MDIChildCount;
  DupsFound := 0;
  for Cntr := 1 to dgTables.RowCount - 1 do
    if NameList.IndexOf(UpperCase(dgTables.Cells[0,Cntr])) = -1 then
      NameList.Add(UpperCase(dgTables.Cells[0,Cntr]))
    else begin
      Inc(DupsFound);
      dgTables.Cells[0,Cntr] := '';
      end;
    {fi}
  {od for}
  sDupsFound := 0;
  for Cntr := 1 to dgTables.RowCount - 1 do
    if NameList.IndexOf(UpperCase(dgTables.Cells[1,Cntr])) = -1 then
      NameList.Add(UpperCase(dgTables.Cells[1,Cntr]))
    else begin
      Inc(sDupsFound);
      dgTables.Cells[1,Cntr] := '';
      end;
    {fi}
  {od for}
  NameList.Free;
  if (DupsFound > 0) or (sDupsFound > 0) then begin
    sTemp := '';
    if DupsFound > 0 then
      sTemp := IntToStr(DupsFound)+' TTable duplicates found';
    {fi}
    if Length(sTemp) > 0 then
      sTemp := sTemp + #13#13;
    {fi}
    if sDupsFound > 0 then
      sTemp := sTemp + IntToStr(sDupsFound)+' TDataSource duplicates found';
    {fi}
    sTemp := sTemp + #13#13 + 'Table entries are blank.';
    MessageDlg(sTemp,mtWarning,[mbOk],0);
    end;
  {fi}
end;  { TformDataMod.LoadTables }

{***********************************************************************}
{                                                                       }
{  SaveTbl is called by TformDataMod.SaveCode and and SavePas to        }
{  generate the code necessary to create a table.                       }
{                                                                       }
{***********************************************************************}

procedure TformDataMod.SaveTbl(aChild: TMDIChild);
var
  Cntr: Integer;
{func/proc}
  function GetType(aType: DbType): string;
  begin
    case aType of
      Lvl5Type: Result := 'ttDBase';
      Lvl7Type: Result := 'ttDBase';
      FoxProType: Result := 'ttFoxPro';
    end;
  end;  { GetType }
  function GetLevel(aType: DbType): string;
  begin
    case aType of
      Lvl5Type: Result := '5';
      Lvl7Type: Result := '7';
      FoxProType: Result := '0';
    end;
  end;  { GetLevel }
  function GetOptions(Options: TIndexOptions): string;
  var
    sTemp: string;
  {func/proc}
    function AddOption(Options,Option: string): string;
    begin
      if Length(Options) > 1 then
        Result := Options + ', '+Option
      else
        Result := Options + Option;
      {fi}
    end;
  begin
    sTemp := '[';
    if ixExpression in Options then sTemp := sTemp + 'ixExpression';
    if ixDescending in Options then sTemp := AddOption(sTemp,'ixDescending');
    if ixUnique in Options then sTemp := AddOption(sTemp,'ixUnique');
    if ixPrimary in Options then sTemp := AddOption(sTemp,'ixPrimary');
    Result := sTemp + ']';
  end;  { GetOptions }
begin  { TformDataMod.SaveTbl }
  with aChild do begin
    WriteLn(F,'  {  Create ',Table1.TableName,'  }');
    WriteLn(F,'  with Table0 do begin');
    WriteLn(F,'    try');
    WriteLn(F,'      DatabaseName := aDbase;');
    WriteLn(F,'      TableName := ''',Table1.TableName,''';');
    WriteLn(F,'      TableType := ',GetType(dBASEtype),';');
    WriteLn(F,'      TableLevel := ',GetLevel(dBASEtype),';');
    { Add Field Defintions }
    WriteLn(F,'      with FieldDefs do begin');
    WriteLn(F,'        Clear;');
    for Cntr := 0 to High(TableStructure) do begin
      with TableStructure[Cntr] do begin
        WriteLn(F,'        with AddFieldDef do begin');
        WriteLn(F,'          Name := ''',Trim(Name),''';');
        if FieldType = '+' then
          WriteLn(F,'          DataType := ftAutoInc;')
        else if FieldType = 'B'then begin
          WriteLn(F,'          DataType := ftTypedBinary;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'C' then begin
          WriteLn(F,'          DataType :=ftString;');
          WriteLn(F,'          Size := ',Length,';');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'D' then begin
          WriteLn(F,'          DataType := ftDate;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'F' then begin
          WriteLn(F,'          { Originally defined as an ''F'' type field }');
          WriteLn(F,'          { Will generate equivalent ''N'' type field }');
          WriteLn(F,'          DataType := ftBcd;');
          WriteLn(F,'          Precision := ',Length,';');
          WriteLn(F,'          Size := ',Decimal,';');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'I' then begin
          WriteLn(F,'          DataType := ftInteger;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'L' then begin
          WriteLn(F,'          DataType :=ftBoolean;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'M' then begin
          WriteLn(F,'          DataType :=ftMemo;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'N' then begin
          WriteLn(F,'          DataType := ftBcd;');
          WriteLn(F,'          Precision := ',Length,';');
          WriteLn(F,'          Size := ',Decimal,';');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'G' then begin
          WriteLn(F,'          DataType :=ftDBaseOle;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = 'O' then begin
          WriteLn(F,'          DataType :=ftFloat;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else if FieldType = '@' then begin
          WriteLn(F,'          DataType :=ftDateTime;');
          if Required = 'Y' then
            WriteLn(F,'          Required := True;');
          {fi}
          end
        else
          WriteLn(F,'          DataType := ftUnknown;');
        end;
      {od with TableStructure}
      WriteLn(F,'          end;');
      end;
    {od for}
    WriteLn(F,'        end;');
    WriteLn(F,'      {od with}');
    { Add Index defintions }
    Table1.IndexDefs.Update;
    if Table1.IndexDefs.Count > 0 then begin
      WriteLn(F,'      IndexDefs.Clear;');
      for Cntr := 0 to Table1.IndexDefs.Count-1 do begin
        WriteLn(F,'      with IndexDefs.AddIndexDef do begin');
        WriteLn(F,'        Name := ''',Table1.IndexDefs[Cntr].Name,''';');
        if Length(Table1.IndexDefs[Cntr].Fields) > 0 then
          WriteLn(F,'        Fields := ''',Table1.IndexDefs[Cntr].Fields,''';')
        else
          WriteLn(F,'        Expression := ''',
              StringReplace(Table1.IndexDefs[Cntr].Expression,'''','"',[rfReplaceAll]),
              ''';');
        {fi}
        WriteLn(F,'        Options := ',
              GetOptions(Table1.IndexDefs[Cntr].Options),';');
        WriteLn(F,'        end;');
        WriteLn(F,'      {od with}');
        end;
      {od for}
      end;
    {fi IndexDefs}
    WriteLn(F,'    except');
    WriteLn(F,'      on E: Exception do begin');
    WriteLn(F,'        MessageDlg(E.Message,mtError,[mbOk],0);');
    WriteLn(F,'        Result := False;');
    WriteLn(F,'        Exit;');
    WriteLn(F,'        end;');
    WriteLn(F,'      {od on}');
    WriteLn(F,'    end; {try}');
    WriteLn(F,'    Result := True;');
    WriteLn(F,'    try');
    WriteLn(F,'      CreateTable;');
    if NeedPatch and not ((MemoBlock = 0) or (MemoBlock = 1024)) then
      if dBASEtype<>FoxProType then
        WriteLn(F,'      PatchMemoFile(Table0,'+IntToStr(MemoBlock)+',True);')
      else
        WriteLn(F,'      PatchMemoFile(Table0,'+IntToStr(MemoBlock)+',False);');
      {fi}
    {fi}
    WriteLn(F,'    except');
    WriteLn(F,'      on E: Exception do begin');
    WriteLn(F,'        MessageDlg(''Error creating table. ''+E.Message,mtError,[mbOk],0);');
    WriteLn(F,'        Result := False;');
    WriteLn(F,'        end;');
    WriteLn(F,'      {od on}');
    WriteLn(F,'    end; {try}');
    WriteLn(F,'    end;');
    WriteLn(F,'  {od with Table0}');
    end;
  {od with}
end;  { TformDataMod.SaveTbl }

{***********************************************************************}
{                                                                       }
{  ScanMemo scans the desktop and test the variable MemoBlock.  If a    }
{  other than 0 or 1024 is found, True is returned; otherwise, False    }
{  is returned.  When True, the procedure PatchMemoFile is included in  }
{  the generated code.                                                  }
{                                                                       }
{  ScanMemo is called by btnOKClick and SaveCode.                       }
{                                                                       }
{***********************************************************************}

function TformDataMod.ScanMemo: Boolean;
var
  Cntr: Integer;
begin
  Result := False;
  for Cntr := 0 to MainForm.MDIChildCount - 1 do
    if not ((TMDIChild(MainForm.MDIChildren[Cntr]).MemoBlock = 0) or
            (TMDIChild(MainForm.MDIChildren[Cntr]).MemoBlock = 1024)) then begin
       Result := True;
       Break;
       end;
    {fi}
  {od for}
end; { TformDataMod.ScanMemo }

{***********************************************************************}
{                                                                       }
{  SavePatch writes the procedure PatchMemoFile to the text file F.     }
{                                                                       }
{  SavePatch is called by btnOKClick and SaveCode.                      }
{                                                                       }
{***********************************************************************}

procedure TformDataMod.SavePatch;
begin
  WriteLn(F,Asterisks);
  WriteLn(F,Spacer);
  WriteLn(F,'{  PatchMemoFile opens the memo field file, changes the block length to }');
  WriteLn(F,'{  aSize, and then rewrites the file using aSize.                       }');
  WriteLn(F,Spacer);
  WriteLn(F,Asterisks);
  WriteLn(F);
  WriteLn(F,'procedure PatchMemoFile(aTable: TTable; aSize: Word; IsDbase: Boolean);');
  WriteLn(F,'var');
  WriteLn(F,'  aName: string;');
  WriteLn(F,'  aBuffer: array of byte;');
  WriteLn(F,'  aFile: File of Byte;');
  WriteLn(F,'begin');
  WriteLn(F,'  aName := aTable.DatabaseName + ''\'' + aTable.TableName;');
  WriteLn(F,'  if IsDbase then');
  WriteLn(F,'    aName := ChangeFileExt(aName,''.DBT'')');
  WriteLn(F,'  else');
  WriteLn(F,'    aName := ChangeFileExt(aName,''.FPT'');');
  WriteLn(F,'  {fi}');
  WriteLn(F,'  AssignFile(aFile,aName);');
  WriteLn(F,'  Reset(aFile);');
  WriteLn(F,'  SetLength(aBuffer,32);');
  WriteLn(F,'  BlockRead(aFile,Pointer(Pointer(aBuffer)^),32);');
  WriteLn(F,'  if IsDbase then begin');
  WriteLn(F,'    aBuffer[6] := Hi(aSize);');
  WriteLn(F,'    aBuffer[7] := Lo(aSize);');
  WriteLn(F,'    end');
  WriteLn(F,'  else begin');
  WriteLn(F,'    aBuffer[20] := Lo(aSize);');
  WriteLn(F,'    aBuffer[21] := Hi(aSize);');
  WriteLn(F,'    end;');
  WriteLn(F,'  {fi}');
  WriteLn(F,'  Rewrite(aFile);');
  WriteLn(F,'  BlockWrite(aFile,Pointer(Pointer(aBuffer)^),aSize);');
  WriteLn(F,'  CloseFile(aFile);');
  WriteLn(F,'  SetLength(aBuffer,0);');
  WriteLn(F,'end; { SavePatch }');
  WriteLn(F);
end;  { TformDataMod.SavePatch; }

{***********************************************************************}
{                                                                       }
{  Called by TMainForm.mSaveCodeClick to create a unit containing the   }
{  code necessary to create copies of the tables currently open on the  }
{  desktop.                                                             }
{                                                                       }
{***********************************************************************}

procedure TformDataMod.SaveCode;
var
  Cntr: Integer;
  Limit: Integer;
{func/proc}
  function FileId(aName: string): string;
  var
    Ptr: Integer;
  begin
    Ptr := Pos('.',aName);
    Result := Copy(aName,1,Ptr-1);
  end;  { FileId }
begin  { TformDataMod.SaveCode }
  AssignFile(F,MainForm.SaveDialog.FileName);
  Rewrite(F);
  WriteLn(F,'unit '+FileId(ExtractFileName(MainForm.SaveDialog.FileName))+';');
  WriteLn(F);
  WriteLn(F,Asterisks);
  WriteLn(F,Spacer);
  WriteLn(F,'{  The following is a function that creates one or more tables in the   }');
  WriteLn(F,'{  folder "aDbase" specified as a parameter.  If the tables are         }');
  WriteLn(F,'{  successfully created and opened, True is returned.  "AOwner" is a    }');
  WriteLn(F,'{  component that is to own Table0 used to define the various tables.   }');
  WriteLn(F,Spacer);
  WriteLn(F,'{  This function may be used as is or the appropriate sections may be   }');
  WriteLn(F,'{  copied into code of the writer''s choice.                             }');
  WriteLn(F,Spacer);
  WriteLn(F,'{  Code generated by dBASEviewer, written by Douglas P. Baird -         }');
  WriteLn(F,'{  dpbaird@sprynet.com  Please give a credit if this code is used.      }');
  WriteLn(F,Spacer);
  WriteLn(F,Asterisks);
  WriteLn(F);
  WriteLn(F,'interface');
  WriteLn(F);
  WriteLn(F,'uses classes;');
  WriteLn(F);
  WriteLn(F,'function CreateTables(aDbase: string; AOwner: TComponent): Boolean;');
  WriteLn(F);
  WriteLn(F,'implementation');
  WriteLn(F);
  WriteLn(F,'uses dbtables, SysUtils, db, Dialogs;');
  WriteLn(F);
  if NeedPatch then
    SavePatch;
  {fi}
  WriteLn(F,'function CreateTables(aDbase: string; AOwner: TComponent): Boolean;');
  WriteLn(F,'var');
  WriteLn(F,'  Table0: TTable;');
  WriteLn(F,'begin');
  WriteLn(F,'  Table0 := TTable.Create(AOwner);');
  WriteLn(F);
  Limit := MainForm.MDIChildCount - 1;
  for Cntr := 0 to Limit do begin
    SaveTbl(TMDIChild(MainForm.MDIChildren[Cntr]));
    if Cntr < Limit then
      WriteLn(F);
    {fi}
    end;
  {od for}
  WriteLn(F,'  Table0.Free;');
  WriteLn(F,'end;');
  WriteLn(F);
  WriteLn(F,'end.');
  CloseFile(F);
end;  { TformDataMod.SaveCode }

{***********************************************************************}
{                                                                       }
{  TListObj.Create is called by TformDataMod.btnOKClick to add an       }
{  object to the string list TableList that is used by the functions    }
{  and procedures within btnOKClick.                                    }
{                                                                       }
{***********************************************************************}

constructor TListObj.Create(aLine: Integer; aChild: TMDIChild);
begin
  inherited Create;
  Line := aLine;
  Child := aChild;
end;

{***********************************************************************}
{                                                                       }
{  btnOKClick generates the data module.  It includes the following     }
{  functions and procedures:                                            }
{    procedure SaveDTI - saves the .DTI file of the data module         }
{    function GetSource - returns the master source for a table, if any }
{    procedure SaveDFM - saves the .DFM file of the data module         }
{    procedure SavePAS - saves the .PAS file of the data module         }
{    procedure PrintMod - prints the structure of the data module       }
{    procedure SaveMod - saves the structure of the data module to a    }
{                        file                                           }
{    function CheckNames - returns True when there are no duplicate     }
{                          names for TTable's and TDataSource's         }
{                                                                       }
{***********************************************************************}

procedure TformDataMod.btnOKClick(Sender: TObject);
var
  sTemp: String;
  TableList: TStringList;
  Cntr: Integer;
  Ptr: Integer;
{func/proc}
  procedure SaveDTI(aFile: string);
  var
    FilePref: string;
    Cntr: Integer;
  begin
    FilePref := edModName.Text+'\<DefaultSession>\<ImpliedDatabase>.';
    AssignFile(F,aFile);
    Rewrite(F);
    WriteLn(F,'[Designer]');
    WriteLn(F,'Version=1');
    WriteLn(F,'Left=192');
    WriteLn(F,'Top=107');
    WriteLn(F,'Width=696');
    WriteLn(F,'Height=480');
    WriteLn(F,'Splitter=185');
    WriteLn(F,'SelectedTab=Components');
    WriteLn(F);
    WriteLn(F,'[Components]');
    WriteLn(F,'Version=1');
    WriteLn(F,'Left=0');
    WriteLn(F,'Top=0');
    WriteLn(F);
    WriteLn(F,'[TreeView]');
    WriteLn(F,'Version=1');
    WriteLn(F,'PrintHeader=1');
    WriteLn(F,'PrintToSingle=1');
    WriteLn(F,'PrintToFile=0');
    WriteLn(F,'PrintBorders=1');
    WriteLn(F,'PrintSelected=0');
    WriteLn(F,'PrintVisible=0');
    Write(F,'ExpandedItems=');
    for Cntr := 0 to TableList.Count-1 do begin
      with TListObj(TableList.Objects[Cntr]) do
        Write(F,FilePref,Child.Table1.DatabaseName,'\',dgTables.Cells[0,Line]);
      {od with}
      if Cntr < TableList.Count-1 then
        Write(F,',')
      else
        WriteLn(F);
      {fi}
      end;
    {od for}
    WriteLn(F);
    WriteLn(F,'[DataDiagrams]');
    WriteLn(F,'Version=1');
    WriteLn(F,'PrintHeader=1');
    WriteLn(F,'PrintToSingle=1');
    WriteLn(F,'PrintToFile=0');
    WriteLn(F,'PrintBorders=1');
    WriteLn(F,'Left=0');
    WriteLn(F,'Top=0');
    WriteLn(F,'ModeSelected=SelectionMode');
    WriteLn(F,'Count=1');
    WriteLn(F);
    WriteLn(F,'[DataDiagrams.Diagram.0]');
    WriteLn(F,'Islands=');
    WriteLn(F,'Comments=');
    WriteLn(F,'Bridges=');
    WriteLn(F,'Alludes=');
    WriteLn(F);
    CloseFile(F);
  end;  { SaveDTI }

  function GetSource(aSource: TDataSource): string;
  var
    Cntr: Integer;
    Found: Integer;
  begin
    Found := -1;
    for Cntr := 0 to TableList.Count-1 do begin
      with TListObj(TableList.Objects[Cntr]).Child do
        if aSource = DataSource1 then begin
          Found := Cntr;
          Break;
          end;
      {od with}
      end;
    {od for}
    if Found > -1 then
      with TListObj(TableList.Objects[Found]) do
        Result := dgTables.Cells[1,Line]
    else
      Result := '';
    {fi}
  end;

{***********************************************************************}
{                                                                       }
{  SaveDFM is driven by constants that control the placement of TTable  }
{  and TDataSource controls.  Each control is placed in a "cell" with a }
{  top and a left offset from either the edge of the area containing    }
{  the controls or of adjacent cells.  The height of a cell is is fixed }
{  by Delphi but the with is determined by either the length of the     }
{  name of the control or the width of the control.  A "column"         }
{  consists of the space for both a TTable Cell and a TDataSource       }
{  control.  A "row" consists of a set of cells across the screen.      }
{                                                                       }
{  SaveDFM sets the number of columns and rows as a function of the     }
{  number of tables and a set of of constants specified in the program. }
{  The width of a cell is calculated from the length of the name of the }
{  control.                                                             }
{                                                                       }
{  The parameters are:                                                  }
{    PanelWidth, PanelHeight - size of panel to display controls        }
{    TopOffset, LeftOffset - spacing                                    }
{    CellWidth, CellHeight - used to calculate layout                   }
{    MinCtlrWidth - use if length of name is smaller                    }
{                                                                       }
{***********************************************************************}

  procedure SaveDFM(aFile: string);
  const
    PanelWidth = 490;
    PanelHeight = 404;
    TopOffset = 8;
    LeftOffset = 8;
    CellWidth = 130;
    CellHeight = 48;
    MinCtrlWidth = 28;
  var
    NmbrCols: Integer;
    Cntr: Integer;
    ColCntr: Integer;
    RowCntr: Integer;
    Multiplier: Single;
    ColOffset: Integer;
  {func/proc}
    function GetLeft(aName: string): string;
    var
      aWidth: Integer;
    begin
      aWidth := Canvas.TextWidth(aName);
      Result := IntToStr(ColOffset+LeftOffset+Max((aWidth-MinCtrlWidth)div 2,0));
      ColOffset := ColOffset + LeftOffset + Max(aWidth,MinCtrlWidth);
    end;
  begin  { SaveDFM }
    Multiplier := (PanelWidth * CellHeight)/(PanelHeight * CellWidth);
    NmbrCols := Ceil(Sqrt(Multiplier*MainForm.MDIChildCount));
    AssignFile(F,aFile);
    Rewrite(F);
    WriteLn(F,'object ',edModName.Text,': T',edModName.Text);
    WriteLn(F);
    WriteLn(F,'  OldCreateOrder = False');
    WriteLn(F,'  Left = 192');
    WriteLn(F,'  Top = 107');
    WriteLn(F,'  Height = 480');
    WriteLn(F,'  Width = 696');
    ColCntr := 0;
    RowCntr := 0;
    ColOffset := LeftOffset;
    for Cntr := 0 to TableList.Count-1 do begin
      with TListObj(TableList.Objects[Cntr]).Child do begin
        WriteLn(F,'  object ',
           dgTables.Cells[0,TListObj(TableList.Objects[Cntr]).Line],': TTable');
        if cbActivate.Checked then
          WriteLn(F,'    Active = True');
        {fi}
        WriteLn(F,'    DatabaseName = ''',Table1.DatabaseName,'''');
        if cbProperties.Checked then begin
          if Length(Table1.Filter) > 0 then begin
            WriteLn(F,'    Filter = ''',Table1.Filter,'''');
            if Table1.Filtered then
              WriteLn(F,'    Filtered = True');
            {fi}
            end;
          {fi Filter}
          if Length(Table1.IndexName) > 0 then
            WriteLn(F,'    IndexName = ''',Table1.IndexName,'''');
          {fi}
          if Length(Table1.MasterFields) > 0 then begin
            WriteLn(F,'    MasterFields = ''',Table1.MasterFields,'''');
            WriteLn(F,'    MasterSource = ',GetSource(Table1.MasterSource));
            end;
          {fi Master}
          end;
        {fi Properties}
        WriteLn(F,'    TableName = ''',Table1.TableName,'''');
        WriteLn(F,'    Left = ',
            GetLeft(dgTables.Cells[0,TListObj(TableList.Objects[Cntr]).Line]));
        WriteLn(F,'    Top = ',IntToStr(TopOffset + CellHeight*RowCntr));
        WriteLn(F,'  end');
        end;
      {od with}
      with TListObj(TableList.Objects[Cntr]) do begin
        WriteLn(F,'  object ',dgTables.Cells[1,Line],': TDataSource');
        WriteLn(F,'    DataSet = ',dgTables.Cells[0,Line]);
        WriteLn(F,'    Left = ',GetLeft(dgTables.Cells[1,Line]));
        WriteLn(F,'    Top = ',IntToStr(TopOffset + CellHeight*RowCntr));
        WriteLn(F,'  end');
        end;
      {od with}
      Inc(ColCntr);
      if ColCntr >= NmbrCols then begin
        ColCntr := 0;
        ColOffset := LeftOffset;
        Inc(RowCntr);
        end;
      {fi}
      end;
    {od for}
    WriteLn(F,'end');
    CloseFile(F);
  end;  { SaveDFM }

  procedure SavePAS(aFile: string);
  var
    sTemp: string;
    Cntr: Integer;
  begin
    sTemp := ExtractFileName(aFile);
    Cntr := Pos('.',sTemp);
    sTemp := Copy(sTemp,1,Cntr-1);
    AssignFile(F,aFile);
    Rewrite(F);
    WriteLn(F,'unit ',sTemp,';');
    WriteLn(F);
    WriteLn(F,Asterisks);
    WriteLn(F,Spacer);
    WriteLn(F,'{  This data module specifies a set of dBASE and/or FoxPro tables that  }');
    WriteLn(F,'{  can be accessed by an application.                                   }');
    WriteLn(F,Spacer);
    if cbCreate.Checked = True then
    WriteLn(F,'{  The following are defined                                            }')
    else if cbOpen.Checked = True then
    WriteLn(F,'{  The following is defined                                             }');
    {fi}
    if cbCreate.Checked = True then begin
      WriteLn(F,Spacer);
      WriteLn(F,'{  function CreateDatabase(aDbase: string; AOwner: TComponent): Boolean;}');
      WriteLn(F,Spacer);
      WriteLn(F,'{    Where "aDbase" is used as the DatabaseName. If the tables are      }');
      WriteLn(F,'{    successfully created and opened, True is returned.  "AOwner" is a  }');
      WriteLn(F,'{    component that is to own Table0 used to define the various tables. }');
      end;
    {fi}
    if cbOpen.Checked = True then begin
      WriteLn(F,Spacer);
      WriteLn(F,'{  function OpenDatabase(aDB: string): Boolean;                         }');
      WriteLn(F,'{    Where "aDB" is used as the DatabaseName.  If the tables are        }');
      WriteLn(F,'{    successfully opened, True is returned.                             }');
      WriteLn(F,Spacer);
      WriteLn(F,'{  procedure CloseDatabase;                                             }');
      WriteLn(F,Spacer);
      WriteLn(F,'{    Sets Active to False in all tables.                                }');
      end;
    {fi}
    if NeedPatch then begin
      WriteLn(F,Spacer);
      WriteLn(F,'{  procedure PatchMemoFile(aTable: TTable; aSize: Word; IsDbase: Boolean)   }');
      WriteLn(F,'{    where aFile is the name of the .DBF file, aSize is the block size  }');
      WriteLn(F,'{    in the memo field file, and IsDbase is True for dBASE files and    }');
      WriteLn(F,'{    False for FoxPro files.  It patches the memo field file header     }');
      WriteLn(F,'{    with the new size and writes a block of that size.                 }');
      end;
    {fi}
    WriteLn(F,Spacer);
    WriteLn(F,'{  Code generated by dBASEviewer, written by Douglas P. Baird -         }');
    WriteLn(F,'{  dpbaird@sprynet.com  Please give a credit if this code is used.      }');
    WriteLn(F,Spacer);
    WriteLn(F,Asterisks);
    WriteLn(F);
    WriteLn(F,'Interface');
    WriteLn(F);
    WriteLn(F,'uses');
    WriteLn(F,'  Windows, Messages, SysUtils, Classes, Graphics, Controls,');
    WriteLn(F,'  Forms, Dialogs, Db, DBTables;');
    WriteLn(F);
    WriteLn(F,'type');
    WriteLn(F,'  T',edModName.Text,' = class(TDataModule)');
    for Cntr := 0 to TableList.Count-1 do
      with TListObj(TableList.Objects[Cntr]) do begin
        WriteLn(F,'    ',dgTables.Cells[0,Line],': TTable;');
        WriteLn(F,'    ',dgTables.Cells[1,Line],': TDataSource;');
      end;
    {od for}
    WriteLn(F,'  private');
    WriteLn(F,'    { Private declarations }');
    if NeedPatch then
      WriteLn(F,'    procedure PatchMemoFile(aTable: TTable; aSize: Word; IsDbase: Boolean);');
    {fi}
    WriteLn(F,'  public');
    WriteLn(F,'    { Public declarations }');
    if cbOpen.Checked then begin
      WriteLn(F,'    function OpenDatabase(aDB: string): Boolean;');
      WriteLn(F,'    procedure CloseDatabase;');
      end;
    {fi}
    if cbCreate.Checked then
      WriteLn(F,'    function CreateDatabase(aDbase: string; AOwner: TComponent): Boolean;');
    {fi}
    WriteLn(F,'  end;');
    WriteLn(F);
    WriteLn(F,'var');
    WriteLn(F,'  ',edModName.Text,': T',edModName.Text,';');
    WriteLn(F);
    WriteLn(F,'implementation');
    WriteLn(F);
    WriteLn(F,'{$R *.DFM}');
    WriteLn(F);
    if cbOpen.Checked then begin
      WriteLn(F,'function T',edModName.Text,'.OpenDatabase(aDB: string): Boolean;');
      WriteLn(F,'{func/proc}');
      WriteLn(F,'  function OpenTbl(aTbl: TTable): Boolean;');
      WriteLn(F,'  begin');
      WriteLn(F,'    aTbl.Active := False;');
      WriteLn(F,'    aTbl.DatabaseName := aDB;');
      WriteLn(F,'    Result := True;');
      WriteLn(F,'    try');
      WriteLn(F,'      aTbl.Active := True;');
      WriteLn(F,'    except');
      WriteLn(F,'      on E: Exception do begin');
      WriteLn(F,'        MessageDlg(E.Message,mtError,[mbOk],0);');
      WriteLn(F,'        Result := False;');
      WriteLn(F,'        end;');
      WriteLn(F,'      {od on}');
      WriteLn(F,'    end;  {try}');
      WriteLn(F,'  end;  {OpenTbl}');
      WriteLn(F,'begin  { OpenDatabase }');
      WriteLn(F,'  Result := True;');
      for Cntr := 0 to TableList.Count-1 do
        with TListObj(TableList.Objects[Cntr]) do begin
          WriteLn(F,'  if not OpenTbl(',dgTables.Cells[0,Line],') then');
          WriteLn(F,'    Result := False;');
          WriteLn(F,'  {fi}');
          end;
        {od with}
      {od for}
      WriteLn(F,'end;  { OpenDatabase }');
      WriteLn(F);
      WriteLn(F,'procedure T',edModName.Text,'.CloseDatabase;');
      WriteLn(F,'begin  { CloseDatabase }');
      for Cntr := 0 to TableList.Count-1 do
        with TListObj(TableList.Objects[Cntr]) do
          WriteLn(F,'  ',dgTables.Cells[0,Line],'.Active := False;');
      {od for}
      WriteLn(F,'end;  { CloseDatabase }');
      WriteLn(F);
      end;
    {fi cbOpen.Checked}
    if cbCreate.Checked then begin
      WriteLn(F,'function T',edModName.Text,
          '.CreateDatabase(aDbase: string; AOwner: TComponent): Boolean;');
      WriteLn(F,'var');
      WriteLn(F,'  Table0: TTable;');
      WriteLn(F,'begin');
      WriteLn(F,'  Result := True;');
      WriteLn(F,'  Table0 := TTable.Create(AOwner);');
      for Cntr := 0 to TableList.Count-1 do begin
        SaveTbl(TListObj(TableList.Objects[Cntr]).Child);
        WriteLn(F);
        end;
      {od for}
      WriteLn(F,'  Table0.Free;');
      WriteLn(F,'end;  { CreateDatabase }');
      WriteLn(F);
      end;
    {fi cbCreateChecked}
    if NeedPatch then begin
      WriteLn(F,'procedure TDataModule1.PatchMemoFile(aTable: TTable; aSize: Word; IsDbase: Boolean);');
      WriteLn(F,'var');
      WriteLn(F,'  aName: string;');
      WriteLn(F,'  aFile: File of Byte;');
      WriteLn(F,'  aBuffer: array of byte;');
      WriteLn(F,'begin');
      WriteLn(F,'  aName := aTable.DatabaseName + ''\'' + aTable.TableName;');
      WriteLn(F,'  if IsDbase then');
      WriteLn(F,'    aName := ChangeFileExt(aName,''.DBT'')');
      WriteLn(F,'  else');
      WriteLn(F,'    aName := ChangeFileExt(aName,''.FPT'');');
      WriteLn(F,'  {fi}');
      WriteLn(F,'  AssignFile(aFile,aName);');
      WriteLn(F,'  Reset(aFile);');
      WriteLn(F,'  SetLength(aBuffer,32);');
      WriteLn(F,'  BlockRead(aFile,Pointer(Pointer(aBuffer)^),32);');
      WriteLn(F,'  if IsDbase then begin');
      WriteLn(F,'    aBuffer[20] := Lo(aSize);');
      WriteLn(F,'    aBuffer[21] := Hi(aSize);');
      WriteLn(F,'    end');
      WriteLn(F,'  else begin');
      WriteLn(F,'    aBuffer[6] := Hi(aSize);');
      WriteLn(F,'    aBuffer[7] := Lo(aSize);');
      WriteLn(F,'    end;');
      WriteLn(F,'  {fi}');
      WriteLn(F,'  Rewrite(aFile);');
      WriteLn(F,'  BlockWrite(aFile,Pointer(Pointer(aBuffer)^),aSize);');
      WriteLn(F,'  CloseFile(aFile);');
      WriteLn(F,'  SetLength(aBuffer,0);');
      WriteLn(F,'end;  { PatchMemoFile }');
      end;
    {fi}
    WriteLn(F,'end.');
    CloseFile(F);
  end;  { SavePAS}

  procedure PrintMod;
  var
    Cntr: Integer;
  begin
    with Prnt do begin
      Start;
      AutomaticPaging := True;
      SetMargins(0.5,0.5,0.5,0.5);
      SetFontInformation('Times New Roman',10,[]);
      SetHeaderDimensions(0.5,0,8.0,0.5,False,0,0);
      SetDetailTopBottom(1.0,0.5);
      SetHeaderInformation(1,wtCenter,0.0,'Structure of '+edModName.Text+
            ' Data Module','Arial',12,[fsBold]);
      SetHeaderInformation(2,wtRight,0.0,'Page %u of %u Pages',
                     'Times New Roman',10,[]);
      SetHeaderInformation(3,wtLeft,0.0,DateToStr(Date),'Times New Roman',10,[]);
      CreateColumn(1,0.5,2);     { Item name }
      CreateColumn(2,2.75,4.75); { Item value }
      CreateColumn(3,0,7.5);     { Header }
      SetTopOfPage;
      for Cntr := 0 to TableList.Count-1 do begin
        WriteColumn(1,wtLeft,wtNextY,'TTable');
        with TListObj(TableList.Objects[Cntr]) do begin
          WriteColumn(2,wtLeft,wtThisY,dgTables.Cells[0,Line]);
          WriteColumn(1,wtLeft,wtNextY,'TDataSource');
          WriteColumn(2,wtLeft,wtThisY,dgTables.Cells[1,Line]);
          end;
        {od with}
        with TListObj(TableList.Objects[Cntr]).Child do begin
          WriteColumn(1,wtLeft,wtNextY,'Database Name');
          WriteColumn(2,wtLeft,wtThisY,Table1.DatabaseName);
          WriteColumn(1,wtLeft,wtNextY,'Table Name');
          WriteColumn(2,wtLeft,wtThisY,Table1.TableName);
          if cbProperties.Checked then begin
            if Length(Table1.Filter) > 0 then begin
              WriteColumn(1,wtLeft,wtNextY,'Filter');
              WriteColumn(2,wtLeft,wtThisY,Table1.Filter);
              WriteColumn(1,wtLeft,wtNextY,'Filtered');
              if Table1.Filtered then
                WriteColumn(2,wtLeft,wtThisY,'True')
              else
                WriteColumn(2,wtLeft,wtThisY,'False');
              {fi}
              end;
            {fi Filter}
            if Length(Table1.IndexName) > 0 then begin
              WriteColumn(1,wtLeft,wtNextY,'IndexName');
              WriteColumn(2,wtLeft,wtThisY,Table1.IndexName);
              end;
            {fi}
            if Length(Table1.MasterFields) > 0 then begin
              WriteColumn(1,wtLeft,wtNextY,'MasterSource');
              WriteColumn(2,wtLeft,wtThisY,GetSource(Table1.MasterSource));
              WriteColumn(1,wtLeft,wtNextY,'MasterFields');
              WriteColumn(2,wtLeft,wtThisY,Table1.MasterFields);
              end;
            {fi Master}
            end;
          {fi Properties}
          end;
        {od with}
        NextLine;
        end;
      {od for}
      Finish;
      end;
    {od with Prnt}
  end;  { PrintMod }

  procedure SaveMod;
  const
    Tab = #9;
  var
    Cntr: Integer;
  begin
    MainForm.SaveDialog.Title := 'Save Data Module Documentation';
    MainForm.SaveDialog.FileName := '';
    MainForm.SaveDialog.DefaultExt := 'txt';
    MainForm.SaveDialog.Filter := 'Text files (*.txt)|*.txt';
    MainForm.SaveDialog.FilterIndex := 1;
    if MainForm.SaveDialog.Execute then begin
      AssignFile(F,MainForm.SaveDialog.FileName);
      Rewrite(F);
      WriteLn(F,'Structure of '+edModName.Text);
      WriteLn(F,DateToStr(Date));
      WriteLn(F);
      for Cntr := 0 to TableList.Count-1 do begin
        with TListObj(TableList.Objects[Cntr]) do begin
          WriteLn(F,'TTable',Tab,dgTables.Cells[0,Line]);
          WriteLn(F,'TDataSource',Tab,dgTables.Cells[1,Line]);
          end;
        {od with}
        with TListObj(TableList.Objects[Cntr]).Child do begin
          WriteLn(F,'Database Name',Tab,Table1.DatabaseName);
          WriteLn(F,'Table Name',Tab,Table1.TableName);
          if cbProperties.Checked then begin
            if Length(Table1.Filter) > 0 then begin
              WriteLn(F,'Filter',Tab,Table1.Filter);
              Write(F,'Filtered',Tab);
              if Table1.Filtered then
                WriteLn(F,'True')
              else
                WriteLn(F,'False');
              {fi}
              end;
            {fi Filter}
            if Length(Table1.IndexName) > 0 then
              WriteLn(F,'IndexName',Tab,Table1.IndexName);
            {fi}
            if Length(Table1.MasterFields) > 0 then begin
              WriteLn(F,'MasterSource',Tab,GetSource(Table1.MasterSource));
              WriteLn(F,'MasterFields',Tab,Table1.MasterFields);
              end;
            {fi Master}
            end;
          {fi Properties}
          end;
        {od with}
        WriteLn(F);
        end;
      {od for}
      CloseFile(F);
      ShowMessage('Documentation Saved');
      end;
    {fi SaveDialog}
  end;  { SaveMod }

  function CheckNames: Boolean;
  var
    Cntr: Integer;
    NameList: TStringList;
  begin
    NameList := TStringList.Create;
    NameList.Capacity := 2 * MainForm.MDIChildCount;
    Result := True;
    for Cntr := 1 to dgTables.RowCount - 1 do begin
      if NameList.IndexOf(UpperCase(dgTables.Cells[0,Cntr])) = -1 then
        NameList.Add(UpperCase(dgTables.Cells[0,Cntr]))
      else begin
        Result := False;
        Break;
        end;
      {fi}
      if NameList.IndexOf(UpperCase(dgTables.Cells[1,Cntr])) = -1 then
        NameList.Add(UpperCase(dgTables.Cells[1,Cntr]))
      else begin
        Result := False;
        Break;
        end;
      {fi}
      end;
    {od for}
    NameList.Free;
  end;  { CheckNames }

begin  { TformDataMod.btnOKClick }
  if CheckNames then begin
    MainForm.SaveDialog.Title := 'Save Data Module';
    MainForm.SaveDialog.FileName := '';
    MainForm.SaveDialog.DefaultExt := 'pas';
    MainForm.SaveDialog.Filter := 'Pascal files (*.pas)|*.pas';
    MainForm.SaveDialog.FilterIndex := 1;
    if MainForm.SaveDialog.Execute then begin
      sTemp := MainForm.SaveDialog.FileName;
      if FileExists(sTemp) then
        if MessageDlg('The file '+ExtractFileName(MainForm.SaveDialog.FileName)+
             ' already exists.  Do you want to replace it?',
             mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
          ModalResult := mrNone;
          Exit;
          end;
        {fi}
      NeedPatch := ScanMemo;
      TableList := TStringList.Create;
      TableList.Sorted := True;
      TableList.Capacity := dgTables.RowCount-1;
      TableList.Clear;
      for Cntr := 1 to dgTables.RowCount-1 do begin
        Ptr := TableList.Add(dgTables.Cells[0,Cntr]);
        TableList.Objects[Ptr] :=
          TListObj.Create(Cntr,TMDIChild(MainForm.MDIChildren[MainForm.MDIChildCount - Cntr]));
        end;
      {od for}
      SaveDTI(ChangeFileExt(MainForm.SaveDialog.FileName,'.dti'));
      SaveDFM(ChangeFileExt(MainForm.SaveDialog.FileName,'.dfm'));
      SavePAS(MainForm.SaveDialog.FileName);
      ShowMessage('Data Module written to '+MainForm.SaveDialog.FileName);
      if cbPrint.Checked then
        PrintMod;
      {fi}
      if cbSave.Checked then
        SaveMod;
      {fi}
      end;
    {fi}
    ModalResult := mrOk;
    for Cntr := 0 to TableList.Count-1 do
      TableList.Objects[Cntr].Free;
    {od for}
    TableList.Free;
    end
  else begin
    MessageDlg('There are duplicate names used for TTable''s and TDataSource''s',
      mtError,[mbOK],0);
    ModalResult := mrNone;
    end;
  {fi}
end;  { TformDataMod.btnOKClick }

{***********************************************************************}
{                                                                       }
{  cbCreateClick toggles the state to include the code to open a new    }
{  database.                                                            }
{                                                                       }
{***********************************************************************}

procedure TformDataMod.cbCreateClick(Sender: TObject);
begin
  if cbCreate.Checked then begin
    SaveOpen := cbOpen.Checked;
    cbOpen.Checked := True;
    cbOpen.Enabled := False;
    end
  else begin
    cbOpen.Checked := SaveOpen;
    cbOpen.Enabled := True;
    end;
  {fi}
end;

end.
