unit DbParmForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, Buttons, ComCtrls, DbTables;

type
  TDbProp = class(TForm)
    StaticText2: TStaticText;
    Label1: TLabel;
    rgFiltered: TRadioGroup;
    GroupBox1: TGroupBox;
    cbCase: TCheckBox;
    cbCompare: TCheckBox;
    memFilter: TMemo;
    Label2: TLabel;
    cbInxName: TComboBox;
    Label3: TLabel;
    cbMstrFields: TComboBox;
    cbMstrSource: TComboBox;
    Label4: TLabel;
    Label5: TLabel;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    edInxFile: TEdit;
    btnView: TBitBtn;
    Label6: TLabel;
    edCallN: TEdit;
    Label7: TLabel;
    procedure btnOKClick(Sender: TObject);
    procedure cbInxNameChange(Sender: TObject);
    procedure cbMstrSourceChange(Sender: TObject);
    procedure btnViewClick(Sender: TObject);
  private
    { Private declarations }
    ThisDatabase: string;
    ThisFileName: TFileName;
    procedure GetFields(aTable: TTable; aField: string);
  public
    { Public declarations }
    procedure Setup;
  end;

var
  DbProp: TDbProp;

implementation

{$R *.DFM}

uses Main, Childwin, Db, IndexDef;

procedure TDbProp.Setup;
var
  BaseName: string;
  Cntr: Integer;
  sTemp: string;
  iTemp: Integer;
  FormSave: TMDIChild;
begin
  cbInxName.ItemIndex := 1;
  cbMstrSource.ItemIndex := 1;
  cbMstrFields.ItemIndex := 1;
  FormSave := MainForm.ActiveMDIChild as TMDIChild;
  if FormSave <> nil then begin
    with FormSave do begin
      { Set local variables }
      ThisDatabase := Table1.DatabaseName;
      ThisFileName := Table1.TableName;
      sTemp := FilePath;
      BaseName := ChangeFileExt(ThisFileName,'');
      edCallN.Text := CallName;
      { Process filter data }
      memFilter.Text := Table1.Filter;
      if Table1.Filtered then
        rgFiltered.ItemIndex := 0
      else
        rgFiltered.ItemIndex := 1;
      {fi}
      cbCase.Checked := foCaseInsensitive in Table1.FilterOptions;
      cbCompare.Checked := foNoPartialCompare in Table1.FilterOptions;
      { Process index files }
      cbInxName.Items.Clear;
      cbInxName.ItemIndex := cbInxName.Items.Add('(None)');
      if  FileExists(ChangeFileExt(sTemp,'.mdx')) or
          FileExists(ChangeFileExt(sTemp,'.cdx'))
         then begin
        Table1.IndexDefs.Update;
        edInxFile.Text := BaseName;
        for Cntr := 0 to Table1.IndexDefs.Count-1 do begin
          sTemp := Table1.IndexDefs.Items[Cntr].Name;
          iTemp := cbInxName.Items.Add(sTemp);
          cbInxName.Items.Objects[iTemp] := Table1.IndexDefs.Items[Cntr];
          if sTemp = Table1.IndexName then
            cbInxName.ItemIndex := iTemp;
          {fi}
          end;
        {od for}
        btnView.Visible := True;
        end { production index exists }
      else begin
        edInxFile.Text := '';
        btnView.Visible := False;
        end;
      {fi FileExists}
      { If there are other files open, setup for links }
      cbMstrSource.Items.Clear;
      cbMstrSource.ItemIndex := cbMstrSource.Items.Add('(None)');
      if  MainForm.MDIChildCount > 1 then begin
        for Cntr := 1 to MainForm.MDIChildCount -1 do begin
          if MainForm.MDIChildren[Cntr] <> FormSave then begin
            with MainForm.MDIChildren[Cntr] as TMDIChild do begin
              {fi}
              iTemp :=  cbMstrSource.Items.Add(CallName);
              cbMstrSource.Items.Objects[iTemp] := DataSource1;
              { If already linked then select item }
              if (FormSave.Table1.MasterSource <> nil) and
                 (DataSource1.DataSet =
                  FormSave.Table1.MasterSource.DataSet) then begin
                cbMstrSource.ItemIndex := iTemp;
                GetFields(Table1,FormSave.Table1.MasterFields);
                end
              else with cbMstrFields do begin
                Items.Clear;
                ItemIndex := Items.Add('[None]');
                Enabled := False;
                end;
              {fi}
              end;
            {od with}
            end;
          {fi}
          end;
        {od for}
        cbMstrSource.Enabled := (cbMstrSource.Items.Count > 1);
        end
      else begin
        cbMstrSource.ItemIndex := 1;
        cbMstrSource.Enabled := False;
        cbMstrFields.ItemIndex := 1;
        cbMstrFields.Enabled := False;
        end;
      {fi}
      end;
    {od with}
    end;
  {fi not nil}
end;  { TDbProp.Setup }

procedure TDbProp.GetFields(aTable: TTable; aField: string);
var
  Cntr: Integer;
begin
  with cbMstrFields do begin
    Items.Clear;
    ItemIndex := Items.Add('(None)');
    for Cntr := 0 to aTable.FieldList.Count - 1 do begin
      Items.Add(aTable.FieldList.Fields[Cntr].FieldName);
      if aField = aTable.FieldList.Fields[Cntr].FieldName then
        ItemIndex := Cntr + 1;
      {fi}
      end;
    {od for}
    Enabled := True;
    end;
  {od with}
end;

procedure TDbProp.btnOKClick(Sender: TObject);
var
  Cntr: Integer;
  Err: Boolean;
begin
  Err := False;
  if MainForm.MDIChildCount > 1 then begin
    for Cntr := 1 to MainForm.MDIChildCount -1 do
      if edCallN.Text = TMDIChild(MainForm.MDIChildren[Cntr]).CallName then begin
        ShowMessage('The call name '+edCallN.Text+' is used by another table.');
        Err := True;
        Break;
        end;
      {fi}
    {od for}
    end;
  {fi}
  if Err then begin
    ModalResult := mrNone;
    Exit;
    end;
  {fi}
  with MainForm.ActiveMDIChild as TMDIChild do begin
    CallName := edCallN.Text;
    SetCaption;
    try
      with Table1 do begin
        Filter := memFilter.Text;
        Filtered := (rgFiltered.ItemIndex = 0) and
                     (Length(memFilter.Text) > 0);
        if cbCase.Checked then
          FilterOptions := [foCaseInsensitive]
        else
          FilterOptions := [];
        {fi}
        if cbCompare.Checked then
          FilterOptions := FilterOptions + [foNoPartialCompare];
        {fi}
        if cbInxName.ItemIndex > 0 then
          IndexName := cbInxName.Text
        else
          IndexName := '';
        {fi}
        with cbMstrSource do
          if (cbInxName.Text <> '(None)') and (ItemIndex > 0) then
            MasterSource := TDataSource(Items.Objects[ItemIndex])
          else
            MasterSource := nil;
          {fi}
        {od with}
        with cbMstrFields do
          if (MasterSource <> nil) and (ItemIndex > 0) then
            MasterFields := Items.Strings[ItemIndex]
          else
            MasterFields := '';
          {fi}
        {od with}
        end;
      {od with Table1}
      dbProp.ModalResult := mrOk;
    except
      on E: Exception do begin
        ShowMessage('Error setting properties. '+E.Message);
        dbProp.ModalResult := mrNone;
        Exit
        end;
      {od on}
    end; {try}
    end;
  {od with ActiveMDIChild}
end;

procedure TDbProp.cbInxNameChange(Sender: TObject);
begin
   cbMstrSource.Enabled := (cbInxName.ItemIndex >= 0);
end;

procedure TDbProp.cbMstrSourceChange(Sender: TObject);
begin
  if cbMstrSource.ItemIndex > 0 then
    with cbMstrSource do
      GetFields(TTable(TDataSource(Items.Objects[ItemIndex]).DataSet),'')
    {od with}
  else
    cbMstrFields.Enabled := False;
  {fi}
end;

procedure TDbProp.btnViewClick(Sender: TObject);
begin
  formIndxDef.LoadTable;
  formIndxDef.ShowModal;
end;

end.
