unit IndexGen;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Grids, DataGrid, Main, ExtCtrls, dbvConst;

type
  TformCreateNdx = class(TForm)
    FieldGrid: TStringGrid;
    btnAdd: TBitBtn;
    btnRemove: TBitBtn;
    btnClear: TBitBtn;
    btnOk: TBitBtn;
    IndexGrid: TDataGrid;
    Panel1: TPanel;
    StaticText1: TStaticText;
    cbExpression: TCheckBox;
    cbDescending: TCheckBox;
    cbUnique: TCheckBox;
    cbPrimary: TCheckBox;
    Label1: TLabel;
    Image1: TImage;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure IndexGridDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure IndexGridDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure FieldGridMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetOptions(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure IndexGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
  private
    { Private declarations }
    DragField: String;
    InSetup: Boolean;
    procedure SetOpts(aRow: Integer);
    procedure RowMove(var msg: TMessage); message RowMoved;
  public
    { Public declarations }
    procedure Setup;
  end;  { TformCreateNdx }

var
  formCreateNdx: TformCreateNdx;

implementation

{$R *.DFM}

uses NewForm;

procedure TformCreateNdx.Setup;
var
  Cntr: Integer;
{func/proc}
  procedure ClearOption(aChar: Char);
  var
    Cntr: Integer;
    Index: Integer;
    sTemp: string;
  begin
    for Cntr := IndexGrid.RowCount-1 downto 1 do begin
      sTemp := IndexGrid.Cells[2,Cntr];
      Index := Pos(aChar,sTemp);
      if Index > 0 then begin
        Delete(sTemp,Index,1);
        IndexGrid.Cells[2,Cntr] := sTemp;
        end;
      {fi}
      end;
    {od for}
  end;
begin  { TformCreateNdx.Setup }
  InSetup := True;
  FieldGrid.RowCount := formNew.DataGrid1.RowCount;
  for Cntr := 1 to formNew.DataGrid1.RowCount-1 do
    FieldGrid.Cells[0,Cntr] := formNew.DataGrid1.Cells[0,Cntr];
  {od for}
  with FieldGrid do
    if Length(Cells[0,RowCount-1]) = 0 then
      RowCount := RowCount-1;
    {fi}
  {od with}
  with FormNew do begin
    case TableType of
      Lvl5Type: begin
        cbUnique.Checked := False;
        cbUnique.Enabled := False;
        cbPrimary.Checked := False;
        cbPrimary.Enabled := False;
        ClearOption('U');
        ClearOption('P');
        end;
      Lvl7Type: begin
        cbUnique.Enabled := True;
        cbPrimary.Enabled := True;
        end;
      FoxProType: begin
        cbUnique.Enabled := True;
        cbPrimary.Checked := False;
        cbPrimary.Enabled := False;
        ClearOption('P');
        end;
      {esac}
    end {case};
    end;
  {od with}
{  IndexGrid.RowCount := 2; }
  IndexGrid.Row := 1;
  IndexGrid.Col := 0;
  SetOpts(1);
  InSetup := False;
end;

procedure TformCreateNdx.FormCreate(Sender: TObject);
begin
  FieldGrid.Cells[0,0] := 'Fields';
  InSetup := False;
end;

procedure TformCreateNdx.IndexGridDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  with IndexGrid do
    if Col = 0 then
      Cells[0,Row] := DragField
    else if Col = 1 then
      if Pos('E',Cells[2,Row]) > 0 then
        Cells[1,Row] := Cells[1,Row] + DragField
      else
        Cells[1,Row] := DragField;
      {fi}
    {fi}
  {od with}
end;

procedure TformCreateNdx.IndexGridDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  CellCoord: TGridCoord;
begin
  with Sender as TStringGrid do begin
    CellCoord := MouseCoord(X,Y);
    if CellCoord.Y > 0 then begin
      Col := CellCoord.X;
      Row := CellCoord.Y;
      SetFocus;
      with Source as TStringGrid do
        DragField := Cells[0,Row];
      {od with}
      Accept := True;
      end;
    {fi}
    end;
  {od with}
end;

procedure TformCreateNdx.FieldGridMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  CellCoord: TGridCoord;
begin
  if Button = mbLeft then
    with Sender as TStringGrid do begin
      CellCoord := MouseCoord(X,Y);
      if CellCoord.Y > 0 then begin
        BeginDrag(False);
        end;
      {fi}
      end;
    {od with}
  {fi}
end;

procedure TformCreateNdx.SetOptions(Sender: TObject);
var
  sTemp: string;
begin
  if not InSetup then begin
    sTemp := '';
    if cbExpression.Checked then sTemp := sTemp + 'E';
    if cbDescending.Checked then sTemp := sTemp + 'D';
    if cbUnique.Checked then sTemp := sTemp + 'U';
    if cbPrimary.Checked then sTemp := sTemp + 'P';
    IndexGrid.Cells[2,IndexGrid.Row] := sTemp;
    end;
  {fi}
end;  { TformCreateNdx.SetOptions }

procedure TformCreateNdx.btnAddClick(Sender: TObject);
begin
  IndexGrid.AppendRow;
  IndexGrid.Row := IndexGrid.RowCount -1;
  cbExpression.Checked := False;
  cbDescending.Checked := False;
  cbUnique.Checked := False;
  cbPrimary.Checked := False;
end;

procedure TformCreateNdx.btnRemoveClick(Sender: TObject);
begin
  with IndexGrid do
    if Row > 1 then
      DeleteRow(Row)
    else
      Rows[1].Clear;
    {fi}
  {od with}
end;

procedure TformCreateNdx.btnClearClick(Sender: TObject);
var
  Cntr: Integer;
begin
  if MessageDlg('No index will be produced.  Do you want to proceed?',
      mtConfirmation,[mbYes,mbNo],0) = mrYes then begin
    for Cntr := IndexGrid.RowCount-1 downto 1 do
      IndexGrid.Rows[Cntr].Clear;
    {od for}
    IndexGrid.RowCount := 2;
    ModalResult := mrNo;
    end;
  {fi}
end;

procedure TformCreateNdx.SetOpts(aRow: Integer);
begin
  with IndexGrid do begin
    cbExpression.Checked := (Pos('E',Cells[2,aRow])>0);
    cbDescending.Checked := (Pos('D',Cells[2,aRow])>0);
    cbUnique.Checked := (Pos('U',Cells[2,aRow])>0);
    cbPrimary.Checked := (Pos('P',Cells[2,aRow])>0);
    end;
  {od with}
end;

procedure TformCreateNdx.IndexGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  CanSelect := True;
  if ARow <> IndexGrid.Row then
    PostMessage(Self.Handle,RowMoved,0,0);
  {fi}
end;

procedure TformCreateNdx.RowMove(var msg: TMessage);
begin
  SetOpts(IndexGrid.Row);
end;  { TformCreateNdx.RowMove }

end.
