{***********************************************************************}
{                                                                       }
{  dBASE Viewer provides the ability to:                                }
{                                                                       }
{    * View one or more XBase files supported by the Borland Database   }
{      Engine.                                                          }
{    * Set table properties to;                                         }
{        - Apply filters                                                }
{        - Set index                                                    }
{        - Link to a master file                                        }
{    * Create dBase Level 7 tables that are:                            }
{        - Are new                                                      }
{        - Clone an existing XBase file                                 }
{    * Print table structure for one table or all open tables           }
{                                                                       }
{  Copyright (c) 2000-2004 Douglas P. Baird                             }
{      email: dougbaird@urgrad.rochester.edu                            }
{                                                                       }
{  Uses the following components not part of Delphi 5:                  }
{                                                                       }
{    * TDataGrid by Alexander Halser, EC Software on Torry's Delphi     }
{      Pages at http://www.torry.net/index.php.                         }
{    * TPrintPreview from PrintPrev by Doug Baird and James "Woody"     }
{      Woodard included as part of this project.                        }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 1.1 12/19/2000                                               }
{                                                                       }
{  Fixed GetCallName to eliminate duplicates from table names with      }
{        numeric suffixes.                                              }
{  Added facilities to display deleted records and undeleted selected   }
{        records.  Based on tips from www.swissdelphicenter.ch          }
{  Added a facility to pack a table.                                    }
{  Added a facility to save table structure(s) as Tab a delimited text  }
{        file.                                                          }
{                                                                       }
{  2/20/2001 Fixed problems opening table in a root folder              }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 2.0 6/9/2003                                                 }
{                                                                       }
{  Added facilities to create dBASE level 5 and FoxPro database files.  }
{  Added the ability to copy records into a cloned table.               }
{  Added facilities to Reindex and ZAP a table.                         }
{  Added the ability to open an entire database (folder).               }
{  Added the ability to generate a Delphi unit to create tables .       }
{  Added several refinements to the user interface.                     }
{  Improved error handling capabilities.                                }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 2.1 6/x/2003                                                 }
{                                                                       }
{  Added capability to store a data module containing the open files.   }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 3.1 9/16/2003                                                }
{                                                                       }
{  Added capabilities to use database aliases and to rename and erase   }
{  tables.                                                              }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 3.2 9/2/2004                                                 }
{                                                                       }
{  Added the capability to open multiple tables from the command line   }
{  and correct some bugs.                                               }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 3.3 11/15/2004                                               }
{                                                                       }
{  Added the capability to set the block length in the memo field file  }
{  and the supporting functions.                                        }
{                                                                       }
{  Fixed problems with Required attribute in Level 7  files when        }
{  creating a new file and switching dBASE types.                       }
{                                                                       }
{  Fixed problems with links when multiple copies of the same named     }
{  table are open.  Problems with DeskSave and DeskLoad in this unit    }
{  and btnOkClick in DataModForm unit.                                  }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 3.4 12/7/2005                                                }
{                                                                       }
{  Fixed DeskSave to permit updating an existing desktop .dtp file.     }
{                                                                       }
{  Blocked the inclusion of declaration of procedure PatchMemoFile in   }
{  unit header when not needed.                                         }
{                                                                       }
{***********************************************************************}

unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, StdActns,
  ActnList, ToolWin, ImgList, Db, DBTables, ChildWin, PrintPrev, dbvConst;

type
  TMainForm = class(TForm)
    { Menu Object }
    MainMenu1: TMainMenu;
      File1: TMenuItem;
        FileNewItem: TMenuItem;
        FileOpenItem: TMenuItem;
        Clone1: TMenuItem;
        FileCloseItem: TMenuItem;
        SaveDesktop1: TMenuItem;
        LoadDesktop1: TMenuItem;
        mOpenDb: TMenuItem;
        PrintStructures1: TMenuItem;
        mSaveStructure: TMenuItem;  { Used to save table structures }
        mSaveCode: TMenuItem;
        mDataMod: TMenuItem;
        mExit: TMenuItem;
        mDelAlias: TMenuItem;
      Database1: TMenuItem;
        Browse1: TMenuItem;
        ListView1: TMenuItem;
        Top1: TMenuItem;
        Prior1: TMenuItem;
        Next1: TMenuItem;
        Last1: TMenuItem;
        Delete1: TMenuItem;
        DbProprties1: TMenuItem;
        Structure1: TMenuItem;
        Edit1: TMenuItem;
        mPackTable: TMenuItem;
        mToggleDeleted: TMenuItem;
        mUndelete: TMenuItem;
        mReindex: TMenuItem;
        mZAP: TMenuItem;
        mRename: TMenuItem;
        mErase: TMenuItem;
      Window1: TMenuItem;
        WindowCascadeItem: TMenuItem;
        WindowTileItem: TMenuItem;
        WindowTileItem2: TMenuItem;
        WindowMinimizeItem: TMenuItem;
        WindowArrangeItem: TMenuItem;
      Help1: TMenuItem;
        HelpContentItem: TMenuItem;
        HelpAboutItem: TMenuItem;
      N1: TMenuItem;
      N2: TMenuItem;
      N3: TMenuItem;
      N4: TMenuItem;
      N5: TMenuItem;
    {Action List }
    ActionList1: TActionList;
      { File menu actions }
        FileNew1: TAction;
        FileExit1: TAction;
        FileOpen1: TAction;
        FileClone1: TAction;
        FileClose1: TWindowClose;
        DeskSave: TAction;
        DeskLoad: TAction;
        FileAlias1: TAction;
      { Database menu actions }
        DbBrowse: TAction;
        DbList: TAction;
        RecordFirst: TAction;
        RecordPrior: TAction;
        RecordNext: TAction;
        RecordLast: TAction;
        RecordDelete: TAction;
        RecordUndelete: TAction;
        DbProperties: TAction;
        DbStructure: TAction;
        FileEdit1: TAction;
        Reindex: TAction;
        ZapTable: TAction;
        PackTable: TAction;
      { Windows menu actions }
        WindowCascade1: TWindowCascade;
        WindowTileHorizontal1: TWindowTileHorizontal;
        WindowTileVertical1: TWindowTileVertical;
        WindowMinimizeAll1: TWindowMinimizeAll;
        WindowArrangeAll1: TWindowArrange;
        WindowNext1: TAction;     { F6 }
        WindowPrevious1: TAction; { Shift+F6 }
      {Help menu actions }
        HelpAbout1: TAction;
    ToolBar1: TToolBar;
      tbFileExit: TToolButton;
      tbSeparator1: TToolButton;
      tbFileNew: TToolButton;
      tbFileOpen1: TToolButton;
      tbFileClone: TToolButton;
      tbFileClose: TToolButton;
      tbFileAlias: TToolButton;
      tbSeparator2: TToolButton;
      tbDbBrowse: TToolButton;
      tbDbList: TToolButton;
      tbSeparator3: TToolButton;
      tbRecordFirst: TToolButton;
      tbRecordPrior: TToolButton;
      tbRecordLast: TToolButton;
      tbRecordNext: TToolButton;
      tbDbProperties: TToolButton;
      tbSeparator4: TToolButton;
      tbDbStructure: TToolButton;
      tbSeparator5: TToolButton;
      tbFileEdit: TToolButton;
      tbDelete: TToolButton;
      tbUndelete: TToolButton;
    { Other objects }
    ImageList1: TImageList;  { List of menu|button images }
    DskTable: TTable;        { Used to save desktop }
    Prn: TPrintPreview;      { used to print table structure(s) }
    OpenDialog: TOpenDialog; { Used to open table & load desktop }
    SaveDialog: TSaveDialog; { Used to create table & save desktop }
    StatusBar: TStatusBar;
    procedure DbBrowseExecute(Sender: TObject);
    procedure DbListExecute(Sender: TObject);
    procedure DbPropertiesExecute(Sender: TObject);
    procedure DbStructureExecute(Sender: TObject);
    procedure DeskLoadExecute(Sender: TObject);
    procedure DeskSaveExecute(Sender: TObject);
    procedure FileClone1Execute(Sender: TObject);
    procedure FileClose1Execute(Sender: TObject);
    procedure FileEdit1Execute(Sender: TObject);
    procedure FileExit1Execute(Sender: TObject);
    procedure FileNew1Execute(Sender: TObject);
    procedure FileOpen1Execute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpAbout1Execute(Sender: TObject);
    procedure PrintStructures1Click(Sender: TObject);
    procedure RecordFirstExecute(Sender: TObject);
    procedure RecordLastExecute(Sender: TObject);
    procedure RecordNextExecute(Sender: TObject);
    procedure RecordPriorExecute(Sender: TObject);
    procedure WindowArrangeAll1Execute(Sender: TObject);
    procedure WindowNext1Execute(Sender: TObject);
    procedure WindowPrevious1Execute(Sender: TObject);
    procedure helpContentItemClick(Sender: TObject);
    procedure RecordDeleteExecute(Sender: TObject);
    procedure mToggleDeletedClick(Sender: TObject);
    procedure RecordUndeleteExecute(Sender: TObject);
    procedure mSaveStructureClick(Sender: TObject);
    procedure ZapTableExecute(Sender: TObject);
    procedure ReindexExecute(Sender: TObject);
    procedure PackTableExecute(Sender: TObject);
    procedure mSaveCodeClick(Sender: TObject);
    procedure mOpenDbClick(Sender: TObject); 
    procedure mDataModClick(Sender: TObject);
    procedure FileAlias1Execute(Sender: TObject);
    procedure mDelAliasClick(Sender: TObject);
    procedure mEraseClick(Sender: TObject);
    procedure mRenameClick(Sender: TObject);
  private
    { Private declarations }
    fOut: TextFile;
    function BuildNew: Boolean;
    procedure CloseDup(var msg: TMessage); message CloseDb;
    function CreateMDIChild(const Name: string; var aType: DbType;
                var aChild: TMDIChild ): Boolean;
    procedure InitialOpen(var msg: TMessage); message OpenDb;
    procedure ShowHint(Sender: TObject);
    function NoDuplicate: Boolean;
    function GetCallNm(aSource: TDataSource): string;
  public
    { Public declarations }
    aFileName: TFileName;  { Used to pass file name to MDI child }
    aFileType: DbType; {Used to pass file type: UnkType,
                         Lvl5Type, Lvl7Type, FoxProType }
    MDICreated: Boolean;
    AliasMode: Boolean;
    procedure PrintStruct(aChild: TMDIChild; First, Last: Boolean);
    function SaveStruct(aChild: TMDIChild; First, Last: Boolean): Boolean;
    procedure SetDbActions(Enable, EMode: Boolean);
    function GetCallName(aName: string): string;
    procedure SetNav;
    function AliasToFile(aName: string): string;
  end;
  function RecIsDeleted(aTable: TTable): Boolean;
  function GetOptions(Options: TIndexOptions): string;
var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses About, DbParmForm, NewForm, IndexGen, TableStructure, bde,
     CopyForm, OpenDbForm, DataModForm, AliasDialog, dbgrids;
     
function GetOptions(Options: TIndexOptions): string;
var
  sTemp: string;
begin
  if ixExpression in Options then sTemp := 'E';
  if ixDescending in Options then sTemp := sTemp + 'D';
  if ixUnique in Options then sTemp := sTemp + 'U';
  if ixPrimary in Options then sTemp := sTemp + 'P';
  Result := sTemp;
end;

function TMainForm.BuildNew: Boolean;
var
  aChild: TMDIChild;
begin  { TMainForm.BuildNew }
  if FormNew.ShowModal = mrOk then begin
    Result := True;
    aFileType := FormNew.TableType;
    CreateMDIChild(aFileName,aFileType,aChild);
    end
  else
    Result := False;
  {fi NewFile}
end;  { TMainForm.BuildNew }

procedure TMainForm.CloseDup(var msg: TMessage);
begin
  FileClose1Execute(Self);
end;

function TMainForm.GetCallNm(aSource: TDataSource): string;
var
  Cntr: Integer;
begin
  for Cntr := 0 to MainForm.MDIChildCount -1 do
    if TMDIChild(MainForm.MDIChildren[Cntr]).Table1 =
                 TTable(aSource.DataSet) then begin
      Result := TMDIChild(MainForm.MDIChildren[Cntr]).CallName;
      Break;
      end
    {if}
  {od for}
end;  { TMainForm.GetCallNm }

function TMainForm.AliasToFile(aName: string): string;
var
  anAlias: string;
  aTable: string;
  aPos: Integer;
  aDb: string;
  aList: TStringList;
begin
  aList := TStringList.Create;
  aPos := Pos('\',aName);
  anAlias := Copy(aName,1,aPos-1);
  aTable := Copy(aName,aPos,MaxInt);
  try
    Session.GetAliasParams(anAlias,aList);
    aPos := Pos('=',aList.Strings[0]);
    aDb := Copy(aList.Strings[0],aPos+1,MaxInt);
  finally
    aList.Free;
  end;
  Result := aDb + aTable;
end;  { TMainForm.AliasToFile }

{***********************************************************************}
{  CreateMDIChild creates a child window using the dBASE table named    }
{  Name of type aType.  If aType is UnkType then the header is checked  }
{  and the type determined.  It is called by:                           }
{      BuildNew - also called by FileClone1Execute                     }
{      DeskLoadExecute                                                  }
{      FileOpen1Execute                                                 }
{      InitialOpen                                                      }
{  FileOpen1Execute and InitialOpen specify UnkType while the other     }
{  routines speciify Lvl7Type, Lvl7Type or FoxProType.                  }
{***********************************************************************}

function TMainForm.CreateMDIChild(const Name: string; var aType: DbType;
             var aChild: TMDIChild): Boolean;
var
  aFile: file of byte;
  aBuffer: array{[0..48]} of byte;
  sTemp: string;
  AmtRead: Integer;
  HeaderSize: Integer;
  RecordSize: Integer;
  BufCntr: Integer;
  aName: string;
{func/proc}
  function NoIndex(aName: String): Boolean;
  begin
    Result := False;
    if MessageDlg('The production index for table '+aName+' does not exist.'+
         #13#13'Press F1 for more information'#13#13'Do you wish to continue?',
         mtWarning,[mbYes,mbNo],55)
         = mrYes then
      if MessageDlg('This will permanently modify the table '+aName+'.'+
         'Are you sure that you want to continue?',mtConfirmation,[mbYes,mbNo],0)
         = mrYes then begin
        Result := True;
        aBuffer[28] := $00;
        Seek(aFile,0);
        BlockWrite(aFile,Pointer(Pointer(aBuffer)^),29);
        end;
      {fi}
    {fi}
  end;
begin  { TMainForm.CreateMDIChild }
  if AliasMode then
    aName := AliasToFile(Name)
  else
    aName := Name;
  {fi}
  if FileExists(aName) then begin
    SetLength(aBuffer,29);
    AssignFile(aFile,aName);
    Reset(aFile);
    { Read from table file header }
    BlockRead(aFile,Pointer(Pointer(aBuffer)^),29,AmtRead);
    if AmtRead < 29 then
      aBuffer[0] := $00;
    {fi}
    if aType = UnkType then begin
      if aBuffer[0] = $F5 then begin  { FoxPro file with memo field }
        aType := FoxProType;
        sTemp := ChangeFileExt(aName,'.FPT');
        if not FileExists(sTemp) then begin
          ShowMessage('The memo file '+sTemp+' is missing.');
          Result := False;
          Exit;
          end;
        {fi}
        sTemp := ChangeFileExt(aName,'.CDX');  { Check for index file }
        if (aBuffer[28] = $01) and not FileExists(sTemp) then begin
          Result := NoIndex(Name);
          if not Result then
            Exit;
          {fi}
          end;
        {fi}
        end
      {fi FoxPro}
      else if (aBuffer[0] and $04) <> $00 then begin { dBASE Lvl 7}
        aType := Lvl7Type;
        if aBuffer[0] = $8C then begin
          sTemp := ExtractFileExt(aName);
          sTemp := Copy(sTemp,1,3) + 'T';
          sTemp := ChangeFileExt(aName,sTemp);
          if not FileExists(sTemp) then begin
            ShowMessage('The memo file '+sTemp+' is missing.');
            Result := False;
            Exit;
            end;
          {fi}
          end;
        {fi $8C}
        sTemp := ChangeFileExt(aName,'.MDX');  { Check for index file }
        if (aBuffer[28] = $01) and not FileExists(sTemp) then begin
          Result := NoIndex(aName);
          if not Result then
            Exit;
          {fi}
          end;
        {fi}
        end
      {fi Lvl7}
      else if (aBuffer[0] and $88) <> $00 then begin { dBASE Ver 5 w/Memo file }
        aType := Lvl5Type;
        sTemp := ChangeFileExt(aName,'.DBT');
        if not FileExists(sTemp) then begin
          ShowMessage('The memo file '+sTemp+' is missing.');
          Result := False;
          Exit;
          end;
        {fi}
        sTemp := ChangeFileExt(aName,'.MDX');  { Check for index file }
        if (aBuffer[28] = $01) and not FileExists(sTemp) then begin
          Result := NoIndex(Name);
          if not Result then
            Exit;
          {fi}
          end;
        {fi}
        end
      {fi Lvl5 w/memo}
      else if (aBuffer[0] = $03) and (aBuffer[28] = $01) then begin
        if FileExists(ChangeFileExt(aName,'.MDX')) then
          aType := Lvl5Type
        else if FileExists(ChangeFileExt(aName,'.CDX')) then
          aType := FoxProType
        else begin
          Result := NoIndex(aName);
          if not Result then
            Exit;
          {fi}
          end;
        {fi}
        end
      else if aBuffer[0] = $03 then  { This could have been generated }
        aType := Lvl5Type            { as a FoxPro table.             }
      else begin
        ShowMessage('The format of '+aName+' is not supported.');
        Result := False;
        Exit;
        end;
      {fi}
      { Test validity of the header }
      end;
    {fi type unknown}
    end
  else begin
    ShowMessage('The file named '+aName+' does not exist.');
    Result := False;
    Exit;
    end;
  {fi file exist}
  { Check validity of the header }
  HeaderSize := 256*aBuffer[9] + aBuffer[8];
  SetLength(aBuffer,HeaderSize);
  Seek(aFile,0);
  BlockRead(aFile,Pointer(Pointer(aBuffer)^),HeaderSize,AmtRead);
  CloseFile(aFile);
  if AmtRead < HeaderSize then begin
    ShowMessage('Header format error in table '+aName);
    Result := False;
    Exit;
    end;
  {fi}
  RecordSize := 256*aBuffer[11] + aBuffer[10] - 1;  { Ignore delete byte }
  if aType = Lvl7Type then
    BufCntr := 68
  else
    BufCntr := 32;
  {fi}
  while aBuffer[BufCntr] <> $0D do begin
    if aType = Lvl7Type then begin
      Dec(RecordSize,aBuffer[BufCntr+33]);
      Inc(BufCntr,48);
      end
    else begin
      Dec(RecordSize,aBuffer[BufCntr+16]);
      Inc(BufCntr,32);
      end;
    if BufCntr > HeaderSize then begin
      ShowMessage('Header format error in table '+aName);
      Result := False;
      Exit;
      end;
    {fi}
    end;
  {od while}
  if RecordSize <> 0 then begin
    ShowMessage('Header format error in table '+aName);
    Result := False;
    Exit;
    end;
  {fi}
  { create a new MDI child window }
  aFileName := Name;
  aChild := TMDIChild.Create(Application);
  Result := MDICreated;
end;  { TMainForm.CreateMDIChild }

procedure TMainForm.DbBrowseExecute(Sender: TObject);
begin  { TMainForm.DbBrowseExecute }
  if MDIChildCount > 0 then begin
    DbBrowse.Checked := True;
    DbList.Checked := False;
    with ActiveMDIChild as TMDIChild do begin
      StringGrid1.Visible := False;
      DbGrid1.Visible := True;
      DBGrid1.SetFocus;
      DbGrid1.SelectedIndex := StringGrid1.Row;
      BrowseMode := True;
      end;
    {od with}
    end;
  {fi}
end;  { TMainForm.DbBrowseExecute }

procedure TMainForm.DbListExecute(Sender: TObject);
begin  { TMainForm.DbListExecute }
  if MDIChildCount > 0 then begin
    DbBrowse.Checked := False;
    DbList.Checked := true;
    with ActiveMDIChild as TMDIChild do begin
      StringGrid1.Visible := True;
      DbGrid1.Visible := False;
      StringGrid1.SetFocus;
      ListFields;
      StringGrid1.Row := DbGrid1.SelectedIndex;
      BrowseMode := False;
      end;
    {od with}
    end;
  {fi}
end;  { TMainForm.DbListExecute }

procedure TMainForm.DbPropertiesExecute(Sender: TObject);
begin  { TMainForm.DbPropertiesExecute }
  dbProp.Setup;
  dbProp.ShowModal
end;  { TMainForm.DbPropertiesExecute }

procedure TMainForm.DbStructureExecute(Sender: TObject);
begin  { TMainForm.DbStructureExecute }
  FormStruct.Setup(TMDIChild(ActiveMDIChild));
  FormStruct.ShowModal;
end;  { TMainForm.DbStructureExecute }

procedure TMainForm.DeskLoadExecute(Sender: TObject);
var
  SaveDir, SaveExt, SaveFilter: string;
  Cntr: Integer;
  Links: Boolean;
  aSourceName: string;
  aSource: TDataSource;
  aChild: TMDIChild;
  sTemp: string;
  SaveMode: Boolean;
  MasterChild: Integer;
  aColumn: TColumn;
  aFieldList: TStringList;
{func/proc}
  function GetFName: string;
  var
    sTemp: string;
  begin
    with MainForm.ActiveMDIChild as TMDIChild do begin
      sTemp := Table1.DatabaseName;
      if sTemp[Length(sTemp)] <> '\' then
        sTemp := sTemp + '\' + Table1.TableName
      else
        sTemp := sTemp + Table1.TableName;
      {fi}
      end;
    {od wiith}
    Result := sTemp;
  end;
begin  { TMainForm.DeskLoadExecute }
  SaveMode := AliasMode;
  if MDIChildCount = 0 then begin
    SaveDir := OpenDialog.InitialDir;
    SaveExt := OpenDialog.DefaultExt;
    SaveFilter := OpenDialog.Filter;
    OpenDialog.Title := 'Load Desktop';
    OpenDialog.InitialDir := OpenDialog.InitialDir;
    OpenDialog.FileName := '';
    OpenDialog.DefaultExt := 'dtp';
    OpenDialog.Filter := 'Desktop Open Files (.dtp)|*.DTP';
    OpenDialog.FilterIndex := 1;
    Links := False;
    if OpenDialog.Execute then begin
      aFieldList := TStringList.Create;
      DskTable.DatabaseName := ExtractFileDir(OpenDialog.FileName);
      DskTable.TableName := ExtractFileName(OpenDialog.FileName);
      try
        DskTable.Active := True;
        DskTable.First;
        if DskTable.FieldValues['WinState'] = wsNormal then begin
          MainForm.Top := DskTable.FieldValues['Top'];
          MainForm.Left := DskTable.FieldValues['Left'];
          MainForm.Height := DskTable.FieldValues['Height'];
          MainForm.Width := DskTable.FieldValues['Width'];
          end;
        {fi}
        MainForm.WindowState := DskTable.FieldValues['WinState'];
        DskTable.Next;
        repeat
{        while not DskTable.Eof do begin }
          case DskTable.FieldByName('RecType').AsInteger of
            2: begin
              aFileType := DskTable['LvlType'];
              sTemp := DskTable.FieldValues['TblName'];
              AliasMode := (sTemp[2] <> ':');
              if CreateMDIChild(sTemp,aFileType,aChild)then begin
                with ActiveMDIChild as TMDIChild do begin
                  if DskTable.FieldValues['WinState'] = wsNormal then begin
                    Top := DskTable.FieldValues['Top'];
                    Left := DskTable.FieldValues['Left'];
                    Height := DskTable.FieldValues['Height'];
                    Width := DskTable.FieldValues['Width'];
                    end;
                  {fi}
                  WindowState := DskTable.FieldValues['WinState'];
                  CallName := DskTable.FieldValues['CallName'];
                  SetCaption;
                  Table1.Filter := TMemoField(DskTable.FieldByName('Filter')).Value;
                  Table1.Filtered := DskTable.FieldValues['Filtered'];
                  Table1.IndexName := DskTable.FieldByName('IndexName').AsString;
                  Links := Links Or (Length(DskTable.FieldByName('MasterSource').AsString) > 0);
                  with Table1.FieldList do begin
                    FieldCnt := Count;
                    StringGrid1.RowCount := FieldCnt;
                    end;
                  {od with}
                  if aFieldList.Capacity < FieldCnt then
                    aFieldList.Capacity := FieldCnt;
                  {fi}
                  aFieldList.Text :=
                      TMemoField(DskTable.FieldByName('FieldLayout')).Value;
                  for Cntr := 0 to FieldCnt-1 do begin
                    with DBGrid1.Columns do begin
                      aColumn := Add;
                      aColumn.FieldName := aFieldList.Names[Cntr];
                      aColumn.Width :=
                          StrToInt(aFieldList.Values[aFieldList.Names[Cntr]]);
                      end;
                    {od with}

                    end;
                  {od for}
                  aFieldList.Clear;
                  end;
                {od with}
                end
              else if aChild <> nil then
                try
                  aChild.Close;
                except

                end;
              {fi}
              end;
          end; {case}
          until not DskTable.FindNext;
{          end;}
        {od while}
        if Links then begin
          DskTable.First;
          DskTable.Next;
          Cntr := 0;
          while not DskTable.Eof do begin
            if DskTable['RecType'] = 2 then begin
              Inc(Cntr);
              MasterChild := DskTable['MasterForm'];
              if MasterChild >= 0 then begin
                with MDIChildren[MasterChild] as TMDIChild do begin
                  aSourceName := CallName;
                  aSource := DataSource1;
                  end;
                {od with}
                if aSourceName = DskTable.FieldByName('MasterSource').Value then
                  with MDIChildren[MDIChildCount - Cntr]
                                    as TMDIChild do begin
                    Table1.MasterSource := aSource;
                    Table1.MasterFields := TMemoField(DskTable.FieldByName('MasterField')).Text;
                    end
                  {od with}
                else
                  ShowMessage('Unable to set master source for DskTable for ' +
                         DskTable['TblName']);
                {fi}
                end;
              {fi Cntr > 0}
              end;
            {fi}
            DskTable.Next;
            end;
          {od while}
          end;
        DskTable.Close{Active := False};
        SetDbActions(True,False);
      except
        ShowMessage('The file does not exist or is busy.')
      end {try/excpet};
      end;
    {fi Open}
    OpenDialog.InitialDir := SaveDir;
    OpenDialog.DefaultExt := SaveExt;
    OpenDialog.Filter := SaveFilter;
    end
  else
    ShowMessage('Desktop not empty.');
  {fi}
  AliasMode := SaveMode;
end;  { TMainForm.DeskLoadExecute }

procedure TMainForm.DeskSaveExecute(Sender: TObject);
var
  Cntr: Integer;
  ColCntr: Integer;
  aFieldList: TStringList;
{func/proc}
  procedure GetSource(aSource: TDataSource);
  var
    Cntr: Integer;
  begin
    for Cntr := 0 to MDIChildCount - 1 do
      with MDIChildren[Cntr] as TMDIChild do
        if DataSource1 = aSource then begin
          with Table1 do
            TMemoField(DskTable.FieldByName('MasterSource')).Value := CallName;
          {od with}
          DskTable.FieldValues['MasterForm'] := Cntr;
          Break;
          end;
        {fi}
      {od with}
    {od for}
  end;
  procedure FixMBlock(aName: string; aSize: Word);
  var
    aFile: file of Byte;
    aBuffer: array of Byte;
  begin
    AssignFile(aFile,aName);
    Reset(aFile);
    SetLength(aBuffer,64);
    BlockRead(aFile,Pointer(Pointer(aBuffer)^),64);
    aBuffer[20] := Lo(aSize);
    aBuffer[21] := Hi(aSize);
    Rewrite(aFile);
    BlockWrite(aFile,Pointer(Pointer(aBuffer)^),aSize);
    CloseFile(aFile);
    SetLength(aBuffer,0);
  end;  {FixMBlock}
begin  { TMainForm.DeskSaveExecute }
  if MDIChildCount > 0 then begin
    SaveDialog.Title := 'Save Desktop';
    SaveDialog.InitialDir := OpenDialog.InitialDir;
    SaveDialog.FileName := '';
    SaveDialog.DefaultExt := 'dtp';
    SaveDialog.Filter := 'Desktop Save Files (.dtp)|*.DTP';
    SaveDialog.FilterIndex := 1;
    if SaveDialog.Execute then begin
      if FileExists(SaveDialog.FileName) then
        if MessageDlg('The file '+ExtractFileName(SaveDialog.FileName)+
             ' already exists.  Do you want to replace it?',
             mtConfirmation,[mbYes,mbNo],0) = mrNo then
          Exit;
        {fi}
      {fi}
      DskTable.DatabaseName := ExtractFileDir(SaveDialog.FileName);
      DskTable.TableName := ExtractFileName(SaveDialog.FileName);
      if not DskTable.Exists then
        DskTable.CreateTable
      else
        DskTable.EmptyTable;
      {fi}
      FixMBlock(ChangeFileExt(SaveDialog.FileName,'.DTT'),128);
      DskTable.Active := True;
      DskTable.Append;
      DskTable.FieldValues['RecType'] := 1;
      DskTable.FieldValues['Top'] := MainForm.Top;
      DskTable.FieldValues['Left'] := MainForm.Left;
      DskTable.FieldValues['Height'] := MainForm.Height;
      DskTable.FieldValues['Width'] := MainForm.Width;
      DskTable.FieldValues['WinState'] := MainForm.WindowState;
      DskTable.FieldValues['Filtered'] := False;
      DskTable.FieldValues['IndexName'] := '';
      DskTable.FieldValues['LvlType'] := UnkType;
      DskTable.FieldValues['MasterField'] := '';
      DskTable.Post;
      aFieldList := TStringList.Create;
      for Cntr := MDIChildCount - 1 downto 0 do
        with MDIChildren[Cntr] as TMDIChild do begin
          DskTable.Append;
          DskTable.FieldValues['RecType'] := 2;
          TMemoField(DskTable.FieldByName('TblName')).Value := TablePath;
          DskTable.FieldValues['Top'] := Top;
          DskTable.FieldValues['Left'] := Left;
          DskTable.FieldValues['Height'] := Height;
          DskTable.FieldValues['Width'] := Width;
          DskTable.FieldValues['WinState'] := WindowState;
          TMemoField(DskTable.FieldByName('CallName')).Value := CallName;
          TMemoField(DskTable.FieldByName('Filter')).Value := Table1.Filter;
          DskTable.FieldValues['Filtered'] := Table1.Filtered;
          DskTable.FieldValues['IndexName'] := Table1.IndexName;
          DskTable.FieldValues['LvlType'] := dBASEtype;
          if Table1.MasterSource = nil then begin
            DskTable.FieldValues['MasterForm'] := -1;
            end
          else
            GetSource(Table1.MasterSource);
          {fi}
          DskTable.FieldValues['MasterField'] := Table1.MasterFields;
          if aFieldList.Capacity < DBGrid1.Columns.Count then
            aFieldList.Capacity := DBGrid1.Columns.Count;
          {fi};
          for ColCntr := 0 to DBGrid1.Columns.Count -1 do
            with DBGrid1.Columns.Items[ColCntr] do
              aFieldList.Add(FieldName+'='+IntToStr(Width));
            {od with}
          {od for}
          DskTable.FieldValues['FieldLayout'] := aFieldList.Text;
          aFieldList.Clear;
          DskTable.Post;
          end;
        {od with}
      {od for}
      aFieldList.Free;
      DskTable.Active := False;
      ShowMessage('Desktop Saved as '+ SaveDialog.FileName);
      end;
    {fi}
    end
  else
    ShowMessage('Nothing to save');
  {fi}
end;  { TMainForm.DeskSaveExecute }

procedure TMainForm.FileClone1Execute(Sender: TObject);
var
  InChild: TMDIChild;
  OutChild: TMDIChild;
  FormCopy: TFormCopy;
begin  { TMainForm.FileClone1Execute }
  InChild := TMDIChild(ActiveMDIChild);
  FormNew.LoadLists(InChild);
  if BuildNew then begin
    OutChild := TMDIChild(ActiveMDIChild);
    if (InChild.Table1.RecordCount > 0) and
       (MessageDlg('Do you want to copy data from ' +
           InChild.Table1.TableName + ' to ' +
           OutChild.Table1.TableName + '?',mtConfirmation,[mbYes,mbNo],0)
           = mrYes) then begin
      Application.CreateForm(TFormCopy, FormCopy);
      FormCopy.CopyLists(InChild,OutChild);
      FormCopy.ShowModal;
      FormCopy.Free;
      end;
    {fi}
    SetNav;
    end;
  {fi}
end;  { TMainForm.FileClone1Execute }

procedure TMainForm.FileClose1Execute(Sender: TObject);
begin  { TMainForm.FileClose1Execute }
  ActiveMDIChild.Close;
  if MDIChildCount = 1 then begin
    SetDbActions(False,False);
    end;
  {fi}
end;  { TMainForm.FileClose1Execute }

procedure TMainForm.FileEdit1Execute(Sender: TObject);
begin  { TMainForm.FileEdit1Execute }
  with ActiveMDIChild as TMDIChild do begin
    EditMode := not EditMode;
    RecordUndelete.Visible := EditMode and ShowDeleted;
    RecordUndelete.Enabled := EditMode and RecIsDeleted(Table1);
    FileEdit1.Checked := EditMode;
    RecordDelete.Visible := EditMode;
    ZapTable.Visible := EditMode;
    Table1.Active := False;
    Table1.ReadOnly := not EditMode;
    Table1.Active := True;
    if EditMode then
      DBGrid1.Color := clInfoBk
    else
      DBGrid1.Color := clWindow;
    {fi}
    DBGrid1.ReadOnly := not EditMode;
    if ShowDeleted then begin
      Table1.DisableControls;
      try
        Check(DbiSetProp(hDBIObj(Table1.Handle),curSOFTDELETEON,
              LongInt(ShowDeleted)));
      finally
        Table1.EnableControls;
      end;
      Table1.Refresh;
      end;
    {fi}
    end;
  {od with}
  if not DbBrowse.Checked then
    DbBrowseExecute(Sender);
  {fi}
end;  { TMainForm.FileEdit1Execute }

procedure TMainForm.FileExit1Execute(Sender: TObject);
begin  { TMainForm.FileExit1Execute }
  Close;
end;  { TMainForm.FileExit1Execute }

procedure TMainForm.FileNew1Execute(Sender: TObject);
begin
  FormNew.ClearLists;
  BuildNew;
end;

procedure TMainForm.FileOpen1Execute(Sender: TObject);
var
  aChild: TMDIChild;
  aFileName: string;
  Continue: Boolean;
begin  { TMainForm.FileOpen1Execute }
  aChild := nil;
  Continue := False;
  if AliasMode then begin
    AliasForm.DlgMode := Open;
    if AliasForm.ShowModal = mrOk then begin
      Continue := True;
      aFileName := AliasForm.cbAlias.Text+'\'+AliasForm.edFile.Text;
      end;
    {fi}
    end
  else begin
    OpenDialog.Title := 'Open Table';
    OpenDialog.FileName := '';
    OpenDialog.DefaultExt := 'dbf';
    OpenDialog.Filter := 'dBase|*.dbf';
    OpenDialog.FilterIndex := 1;
    if OpenDialog.Execute then begin
      Continue := True;
      aFileName := OpenDialog.FileName;
      end;
    {fi}
    end;
  {fi AliasMode}
  if Continue then begin
    aFileType := UnkType;
    if CreateMDIChild(aFileName,aFileType,aChild)then begin
      if not AliasMode then
        OpenDialog.InitialDir := ExtractFileDir(OpenDialog.FileName);
      {fi}
      end
    else if aChild <> nil then
      try
        aChild.Close;
      except

      end;
    {fi}
    end;
  {fi}
end;  { TMainForm.FileOpen1Execute }

procedure TMainForm.FormCreate(Sender: TObject);
var
  sTemp: string;
begin  { TMainForm.FormCreate }
  Caption := MainCaption;
  Application.OnHint := ShowHint;
  if ParamCount > 0 then begin
    sTemp := ParamStr(1);
    if (Length(sTemp) > 0) and FileExists(sTemp) then begin
      PostMessage(Self.Handle,OpenDb,0,0);
      OpenDialog.InitialDir := ExtractFileDir(sTemp);
      end;
    {fi}
    end;
  {fi}
end;  { TMainForm.FormCreate }

procedure TMainForm.HelpAbout1Execute(Sender: TObject);
begin  { TMainForm.HelpAbout1Execute }
  AboutBox.ShowModal;
end;  { TMainForm.HelpAbout1Execute }

procedure TMainForm.InitialOpen(var msg: TMessage);
var
  Cntr: Integer;
  sTemp: string;
  anExt: string;
  SrchRec: TSearchRec;
  aChild: TMDIChild;
begin  { TMainForm.InitialOpen }
  for Cntr := 1 to ParamCount do begin
    sTemp := ParamStr(Cntr);
    anExt := UpperCase(ExtractFileExt(sTemp));
    if ((anExt = '.DBF')or(anExt = '.DTP')) and FileExists(sTemp) then begin
      {***********************************************************************}
      {  The "Send To" function in Windows Explorer uses the short (8.3)      }
      {  form of file names.  The DBE has problems with that format if the    }
      {  table name has been shortened.  The following code is used to find   }
      {  the long file name for the table.                                    }
      {***********************************************************************}
      FindFirst(sTemp,faAnyFile,SrchRec);
      sTemp := ExtractFileDir(sTemp);
      if Length(sTemp) > 3 then
        sTemp := sTemp + '\';
      {fi}
      aChild := nil;
      aFileType := UnkType;
      if not CreateMDIChild(sTemp + SrchRec.Name,aFileType,aChild) and
         (aChild <> nil) then
        try
          aChild.Close;
        except

        end;
      {fi}
      end
    else
      ShowMessage(sTemp + ' if not a .dbf file.');
    {fi}
    end;
  {od for}
end;  { TMainForm.InitialOpen }

{***********************************************************************}
{                                                                       }
{  PrintStruct prints the structure for one table associated with       }
{  aChild TMDIChild form.                                               }
{                                                                       }
{  Parameters:                                                          }
{    First - First table on report - Starts report definition           }
{    Last  - Last table on report - Finishes report definition and      }
{            initiates print preview.                                   }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.PrintStructures1Click                                    }
{    TFormStruct.btnPrintClick in TableStructure.pas                    }
{                                                                       }
{***********************************************************************}

procedure TMainForm.PrintStruct(aChild: TMDIChild; First, Last: Boolean);
var
  Cntr: Integer;
  Temp: Single;
begin  { TMainForm.PrintStruct }
  with Prn, aChild do begin
    if First then begin
      Start;
      StartPercent := P8_Width;
      AutomaticPaging := True;
      SetMargins(0.5,0.5,0.5,0.5);
      SetFontInformation('Courier New',10,[]);
      CreateColumn(1,0.5,2.5);  { Field or index name}
      CreateColumn(2,3.25,1);   { Field type }
      CreateColumn(3,4.5,1);    { Field Size }
      CreateColumn(4,5.75,1);   { Decimals }
      CreateColumn(5,3.25,3);   { Index value }
      CreateColumn(6,7.0,0.5);  { Required }
      CreateColumn(7,6.75,0.75);{ Index options }
      end
    else if LinesLeft < 5 then
      NewPage;
    {fi}
    WriteLine(wtLeft,wtNextY,'Table Name: '+Table1.TableName);
    NewLines(1);
    WriteColumn(1,wtLeft,wtNextY,'Name');
    WriteColumn(2,wtCenter,wtThisY,'Type');
    WriteColumn(3,wtCenter,wtThisY,'Length');
    WriteColumn(4,wtCenter,wtThisY,'Decimal');
    if aChild.dBASEtype = Lvl7Type then
      WriteColumn(6,wtCenter,wtThisY,'Req''d');
    {fi}
    Temp := TheYPosition + LineHeightInches;
    DrawLine(0.5,Temp,8.0,temp,1);
    for Cntr := 0 to High(TableStructure) do
      with TableStructure[Cntr] do begin
        WriteColumn(1,wtleft,wtNextY,Name);
        WriteColumn(2,wtCenter,wtThisY,FieldType);
        WriteColumn(3,wtCenter,wtThisY,Length);
        WriteColumn(4,wtCenter,wtThisy,Decimal);
        if aChild.dBASEtype = Lvl7Type then
          WriteColumn(6,wtCenter,wtThisY,Required);
        {fi}
        end;
      {od with}
    {od for}
    Table1.IndexDefs.Update;
    if Table1.IndexDefs.Count > 0 then begin
      if LinesLeft < 3 then
        NewPage
      else
        NewLines(1);
      {fi}
      WriteColumn(1,wtLeft,wtNextY,'Index Name');
      WriteColumn(5,wtLeft,wtThisY,'Field/Expression');
      WriteColumn(6,wtLeft,wtThisY,'Options');
      Temp := TheYPosition + LineHeightInches;
      DrawLine(0.5,Temp,8.0,temp,1);
      for Cntr := 0 to Table1.IndexDefs.Count - 1 do
        with Table1.IndexDefs[Cntr] do begin
          WriteColumn(1,wtLeft,wtNextY,Name);
          if ixExpression in Table1.IndexDefs[Cntr].Options then
            WriteColumn(5,wtLeft,wtThisY,FieldExpression)
          else
            WriteColumn(5,wtLeft,wtThisY,Fields);
          {fi}
          WriteColumn(6,wtCenter,wtThisY,
                GetOptions(Table1.IndexDefs[Cntr].Options));
          end;
        {od with}
      {od for}
      end;
    {fi}
    NewLines(1);
    if Table1.DatabaseName[2] = ':' then
      WriteColumn(1,wtLeft,wtNextY,'Database: '+Table1.DatabaseName)
    else
      WriteColumn(1,wtLeft,wtNextY,'Database Alias: '+Table1.DatabaseName);
    {fi}
    WriteColumn(1,wtLeft,wtNextY,'Call Name: '+CallName);
    if Table1.Filtered then
      WriteColumn(1,wtLeft,wtNextY,'Filter: '+Table1.Filter);
    {fi}
    if Length(Table1.IndexName) > 0 then
      WriteColumn(1,wtLeft,wtNextY,'Index: '+Table1.IndexName);
    {fi}
    if Table1.MasterSource <> nil then begin
      WriteColumn(1,wtLeft,wtNextY,'Master Source: '+
                  GetCallNm(Table1.MasterSource));
      WriteColumn(1,wtLeft,wtNextY,'Master Field: '+Table1.MasterFields);
      end;
    {fi}
    if MemoBlock > 0 then
      WriteColumn(1,wtLeft,wtNextY,'Memo Block Size: '+
                  IntToStr(MemoBlock));
    {fi}
    if Last then
      Finish
    else
      NewLines(2);
    {fi}
    end;
  {od with}
end;  { TMainForm.PrintStruct }

{***********************************************************************}
{                                                                       }
{  SaveStruct writes the structure for one table associated with        }
{  aChild TMDIChild form to a file.                                     }
{                                                                       }
{  Parameters:                                                          }
{    First - First table on report - Opens the output file              }
{    Last  - Last table on report - Closes the output file              }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.mSaveStructuresClick                                     }
{    TFormStruct.                                                       }
{                                                                       }
{***********************************************************************}

function TMainForm.SaveStruct(aChild: TMDIChild; First, Last: Boolean): Boolean;
const
  Tab = #9;
  Tabs = #9#9#9;
var
  Cntr: Integer;
  sTemp: string;
begin  { TMainForm.SaveStruct }
  Result := True;
  with aChild do begin
    if First then begin
      SaveDialog.Title := 'Save Structure';
      SaveDialog.FileName := '';
      SaveDialog.DefaultExt := 'txt';
      SaveDialog.Filter := 'Text files (*.txt)|*.txt';
      SaveDialog.FilterIndex := 1;
      if SaveDialog.Execute then begin
        if FileExists(SaveDialog.FileName) then
          if MessageDlg('The file '+ExtractFileName(SaveDialog.FileName)+
               ' already exists.  Do you want to replace it?',
               mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
            Result := False;
            Exit;
            end;
          {fi}
        {fi}
        AssignFile(fOut,SaveDialog.FileName);
        Rewrite(fOut);
        end
      else begin
        Result := False;
        Exit;
        end;
      {fi}
      end;
    {fi}
    WriteLn(fOut,'Table Name: '+Table1.TableName);
    WriteLn(fOut);
    WriteLn(fOut,'Name',Tab,'Type',Tab,'Length',Tab,'Decimal');
    for Cntr := 0 to High(TableStructure) do
      with TableStructure[Cntr] do
        if dBASEtype = Lvl7Type then
          WriteLn(fOut,Name,Tab,FieldType,Tab,Length,Tab,Decimal,Tab,Required)
        else
          WriteLn(fOut,Name,Tab,FieldType,Tab,Length,Tab,Decimal);
        {fi}
      {od with}
    {od for}
    Table1.IndexDefs.Update;
    if Table1.IndexDefs.Count > 0 then begin
      WriteLn(fOut);
      WriteLn(fOut,'Index Name',Tab,'Field/Expression');
      for Cntr := 0 to Table1.IndexDefs.Count - 1 do
        with Table1.IndexDefs[Cntr] do begin
          sTemp := Name + Tab;
          if Length(FieldExpression) > 0 then
            sTemp := sTemp +FieldExpression
          else
            sTemp := sTemp + Fields;
          {fi}
          WriteLn(fOut,sTemp);
          end;
        {od with}
      {od for}
      end;
    {fi}

    WriteLn(fOut);
    if Table1.DatabaseName[2] = ':' then
      WriteLn(fOut,'Database: '+Table1.DatabaseName)
    else
      WriteLn(fOut,'Database Alias: '+Table1.DatabaseName);
    {fi}
    WriteLn(fOut,'Call Name: '+CallName);
    if Table1.Filtered then
      WriteLn(fOut,'Filter: '+Table1.Filter);
    {fi}
    if Length(Table1.IndexName) > 0 then
      WriteLn(fOut,'Index: '+Table1.IndexName);
    {fi}
    if Table1.MasterSource <> nil then begin
      WriteLn(fOut,'Master Source: '+GetCallNm(Table1.MasterSource));
      WriteLn(fOut,'Master Field: '+Table1.MasterFields);
      end;
    {fi}

    if MemoBlock > 0 then
      WriteLn(fOut,'Memo Block Size: '+IntToStr(MemoBlock));
    {fi}

    if Last then
      CloseFile(fOut)
    else
      WriteLn(fOut);
    {fi}
    end;
  {od with}
end;  { TMainForm.SaveStruct }

{***********************************************************************}
{                                                                       }
{  PrintStructures1Click prints the structure of all tables currently   }
{  Open.  Calls TMainForm.PrintStruct.                                  }
{                                                                       }
{***********************************************************************}

procedure TMainForm.PrintStructures1Click(Sender: TObject);
var
  Cntr: Integer;
  Limit: Integer;
  TableList: TStringList;
begin  { TMainForm.PrintStructures1Click }
  if MDIChildCount > 0 then begin
    TableList := TStringList.Create;
    TableList.Sorted := True;
    TableList.Capacity := MDIChildCount;
    Limit := MDIChildCount - 1;
    for Cntr := Limit downto 0 do
      TableList.Objects[TableList.Add(TMDIChild(MDIChildren[Cntr]).CallName)]
          := MDIChildren[Cntr];
    {od for}
    for Cntr := 0 to Limit do
      PrintStruct(TMDIChild(TableList.Objects[Cntr]),Cntr=0,Cntr=Limit);
    {od for}
    TableList.Free;
    end
  else
    ShowMessage('No tables are open.');
  {fi}
end;  { TMainForm.PrintStructures1Click }

procedure TMainForm.RecordFirstExecute(Sender: TObject);
begin  { TMainForm.RecordFirstExecute }
  with ActiveMDIChild as TMDIChild do begin
    Table1.First;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordFirstExecute }

procedure TMainForm.RecordLastExecute(Sender: TObject);
begin  { TMainForm.RecordLastExecute }
  with ActiveMDIChild as TMDIChild do begin
    Table1.Last;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordLastExecute }

procedure TMainForm.RecordNextExecute(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    Table1.Next;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordNextExecute }

procedure TMainForm.RecordPriorExecute(Sender: TObject);
begin  { TMainForm.RecordPriorExecute }
  with ActiveMDIChild as TMDIChild do begin
    Table1.Prior;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordPriorExecute }

{***********************************************************************}
{                                                                       }
{  SetDbActions sets actions for toolbar buttons and menu items to      }
{  reflect existence of at least one open table.                        }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.BuildNew                                                 }
{    TMainForm.BuildNew                                                 }
{    TMainForm.DeskLoadExecute                                          }
{    TMainForm.FileClose1Execute                                        }
{    TMainForm.OpenDbFile                                               }
{                                                                       }
{***********************************************************************}

procedure TMainForm.SetDbActions(Enable, EMode: Boolean);
begin  { TMainForm.SetDbActions }
  FileClose1.Visible := Enable;
  FileClose1.Enabled := Enable;
  RecordFirst.Visible := Enable;
  RecordFirst.Enabled := Enable;
  RecordPrior.Visible := Enable;
  RecordPrior.Enabled := Enable;
  RecordNext.Visible := Enable;
  RecordNext.Enabled := Enable;
  RecordLast.Visible := Enable;
  RecordLast.Enabled := Enable;
  DbProperties.Visible := Enable;
  DbProperties.Enabled := Enable;
  DbStructure.Visible := Enable;
  DbStructure.Enabled := Enable;
  DbBrowse.Visible := Enable;
  DbBrowse.Enabled := Enable;
  DbList.Visible := Enable;
  DbList.Enabled := Enable;
  FileEdit1.Visible := Enable;
  FileEdit1.Enabled := Enable;
  FileClone1.Visible := Enable;
  FileClone1.Enabled := Enable;
  FileClose1.Visible := Enable;
  FileClose1.Enabled := Enable;
  Database1.Visible := Enable;
  Database1.Enabled := Enable;
  SaveDesktop1.Visible := Enable;
  LoadDesktop1.Visible := not Enable;
  RecordDelete.Visible := eMode and Enable;
  Reindex.Visible := eMode and Enable;
  ZapTable.Visible := eMode and Enable;
end; { TMainForm.SetDbActions }

{***********************************************************************}
{                                                                       }
{  WindowNext1Execute activates the next TMDIChild window.  Executed    }
{  with F6                                                              }
{                                                                       }
{***********************************************************************}

procedure TMainForm.WindowNext1Execute(Sender: TObject);
begin  { TMainForm.WindowNext1Execute }
  Next;
end;  { TMainForm.WindowNext1Execute }

{***********************************************************************}
{                                                                       }
{  WindowPrevious1Execute activates the prior TMDIChild window.         }
{  Executed with Shift+F6                                                     }
{                                                                       }
{***********************************************************************}

procedure TMainForm.WindowPrevious1Execute(Sender: TObject);
begin  { TMainForm.WindowNext1Execute }
  Previous;
end;  { TMainForm.WindowNext1Execute }


procedure TMainForm.WindowArrangeAll1Execute(Sender: TObject);
var
  I: Integer;
begin  { TMainForm.WindowArrangeAll1Execute }
  for I := 0 to MDIChildCount - 1 do
    MDIChildren[I].WindowState := wsNormal;
end;  { TMainForm.WindowArrangeAll1Execute }

{***********************************************************************}
{                                                                       }
{  SetNav sets the enables/disables the toolbar database navigator      }
{  buttons and menu items to refelct table status.                      }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.OpenDbFile                                               }
{    TMainForm.RecordFirstExecute                                       }
{    TMainForm.RecordLastExecute                                        }
{    TMainForm.RecordNextExecute                                        }
{    TMainForm.RecordPriorExecute                                       }
{    TMDIChild.FormActivate in Childwin.pas                             }
{                                                                       }
{***********************************************************************}

procedure TMainForm.SetNav;
begin  { TMainForm.SetNav }
  RecordLast.Enabled := not TMDIChild(ActiveMDIChild).Table1.Eof;
  RecordNext.Enabled := not TMDIChild(ActiveMDIChild).Table1.Eof;
  RecordFirst.Enabled := not TMDIChild(ActiveMDIChild).Table1.Bof;
  RecordPrior.Enabled := not TMDIChild(ActiveMDIChild).Table1.Bof;
  with ActiveMDIChild as TMDIChild do
    if StringGrid1.Visible then
      if (StringGrid1.Cells[1,StringGrid1.Row] = '(MEMO)') then
        DBMemo1.DataField := DbGrid1.SelectedField.FieldName
      else
        DBMemo1.Visible := False;
      {fi}
    {fi}
  {od with}
end;  { TMainForm.SetNav }

procedure TMainForm.helpContentItemClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TMainForm.RecordDeleteExecute(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    if MessageDlg('Are you sure that you want to delete this record?',
         mtConfirmation,[mbYes,mbNo],0) = mrYes then
      Table1.Delete;
    {fi}
    end;
  {od with}
end;

{***********************************************************************}
{                                                                       }
{  GetCallName creates a unique identifier for the table being opened   }
{  or created.  It is used when selecting a Master Source in DbParmForm }
{  and is needed to select the correct source when more than one table  }
{  with the same name is open.                                          }
{                                                                       }
{***********************************************************************}
function TMainForm.GetCallName(aName: string): string;
var
  Cntr: Integer;
  sTemp: string;
  aList: TStringList;
{func/proc}
  function GetSuffix(Count: Integer): string;
  begin  { GetSuffix }
    Result := IntToStr(Count);
    {***********************************************************************}
    {  Search for duplicates, incrementing Count until a unique value found.}
    {***********************************************************************}
    while aList.IndexOf(aName+Result) >= 0 do begin
      Inc(Count);
      Result := IntToStr(Count);
      end;
    {od while}
  end;  { GetSuffix }
begin  { TMainForm.GetCallName }
  aList := TStringList.Create;
  for Cntr := 0 to MDIChildCount - 1 do
    with MDIChildren[Cntr] as TMDIChild do begin
      sTemp := Copy(CallName,0,Length(aName));
      if UpperCase(sTemp) = UpperCase(aName) then
        aList.Add(CallName);
      {fi}
      end;
    {od with}
  {od for}
  if aList.Count > 0 then
    Result := aName + GetSuffix(aList.Count)
  else
    Result := aName;
  {fi}
  aList.Free
end;  { TMainForm.GetCallName }

procedure TMainForm.mToggleDeletedClick(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    ShowDeleted := not ShowDeleted;
    RecordUndelete.Visible := ShowDeleted and EditMode;
    if ShowDeleted then
      mToggleDeleted.Caption := '&Hide Deleted'
    else
      mToggleDeleted.Caption := 'S&how Deleted';
    {fi}
    Table1.DisableControls;
    try
      Check(DbiSetProp(hDBIObj(Table1.Handle),curSOFTDELETEON,
            LongInt(ShowDeleted)));
    finally
      Table1.EnableControls;
    end;
    Table1.Refresh;
    if RecIsDeleted(Table1) then begin
      RecordUndelete.Enabled := True;
      StatusBar.Panels[1].Text := 'Record is deleted';
      end
    else begin
      RecordUndelete.Enabled := False;
      StatusBar.Panels[1].Text := '';
      end;
    {fi}
    end;
  {od with}
end;

procedure TMainForm.PackTableExecute(Sender: TObject);
var
  SaveExcl: Boolean;
begin
  with ActiveMDIChild as TMDIChild do begin
    SaveExcl := Table1.Exclusive;
    if not SaveExcl then begin
      Table1.Active := False;
      try
        Table1.Exclusive := True;
      except
        ShowMessage('Cannot get exclusive use of '+Table1.DatabaseName+'\'+
             Table1.TableName+' to pack.');
        Exit;
      end;
      Table1.Active := True;
      end;
    {fi}
    Table1.DisableControls;
    Check(DbiPackTable(Table1.DBHandle,Table1.Handle,nil,
          szDBASE,True));
    Table1.EnableControls;
    Table1.Refresh;
    ShowDeleted := False;
    mToggleDeleted.Caption := 'S&how Deleted';
    if not SaveExcl then begin
      Table1.Active := False;
      Table1.Exclusive := False;
      Table1.Active := True;
      end;
    {fi}
    end;
  {od with}
end;

procedure TMainForm.RecordUndeleteExecute(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    if RecIsDeleted(Table1) then begin
      try
        Check(DbiUndeleteRecord(Table1.Handle));
        MainForm.StatusBar.Panels[1].Text := '';
        MainForm.RecordUndelete.Enabled := False;
        {fi}
      except
        ShowMessage('No undelete performed.');
      end;
      end;
    {fi}
    end;
  {od with}
end;

procedure TMainForm.mSaveStructureClick(Sender: TObject);
var
  Cntr: Integer;
  Limit: Integer;
  TableList: TStringList;
  Done: Boolean;
begin  { TMainForm.mSaveStructuresClick }
  if MDIChildCount > 0 then begin
    TableList := TStringList.Create;
    TableList.Sorted := True;
    TableList.Capacity := MDIChildCount;
    Limit := MDIChildCount - 1;
    for Cntr := Limit downto 0 do
      TableList.Objects[TableList.Add(TMDIChild(MDIChildren[Cntr]).Table1.TableName)]
          := MDIChildren[Cntr];
    {od for}
    Done := True;
    for Cntr := 0 to Limit do
      if not SaveStruct(TMDIChild(TableList.Objects[Cntr]),Cntr=0,Cntr=Limit) then begin
        Done := False;
        Break;
        end;
      {fi}
    {od for}
    if Done then
      ShowMessage('Structure file '+SaveDialog.FileName+' written.');
    {fi}
    TableList.Free;
    end
  else
    ShowMessage('No tables are open.');
  {fi}
end;  { TMainForm.mSaveStructuresClick }

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusBar.Panels[0].Text := Application.Hint;
end;  { TMainForm.ShowHint }

function RecIsDeleted(aTable: TTable): Boolean;
var
  RP: RecProps;
begin
  with aTable do begin
    if (Active = True) and not Bof then begin
      UpdateCursorPos;
      Check(DbiGetRecord(Handle,dbiNOLOCK,nil,@RP));
      Result := RP.bDeleteFlag;
      end
    else
      Result := False;
    end;
  {od with}
end;

procedure TMainForm.ZapTableExecute(Sender: TObject);
begin
  if MessageDlg('All records will be deleted.'#13#13+
       'Are you sure that you want to continue?',
       mtConfirmation,[mbYes,mbNo],0) = mrYes then
    with ActiveMDIChild as TMDIChild do begin
      Table1.DisableControls;
      Table1.Active := False;
      Table1.EmptyTable;
      Table1.Active := True;
      Table1.EnableControls;
      end;
    {od with}
  {fi}
end;

procedure TMainForm.ReindexExecute(Sender: TObject);
var
  IsReadOnly: Boolean;
begin
  with ActiveMDIChild as TMDIChild do begin
    Table1.Active := False;
    Table1.Exclusive := True;
    IsReadOnly := Table1.ReadOnly;
    Table1.ReadOnly := False;
    Table1.Active := True;
    Table1.DisableControls;
    try
      Check(DbiRegenIndexes(Table1.Handle));
    finally
      Table1.EnableControls;
      Table1.Active := False;
      Table1.Exclusive := False;
      Table1.ReadOnly := IsReadOnly;
      Table1.Active := True;
    end {try};
    end;
  {od with}
end;

procedure TMainForm.mSaveCodeClick(Sender: TObject);
begin
  if MDIChildCount > 0 then begin
    SaveDialog.Title := 'Save Code';
    SaveDialog.FileName := '';
    SaveDialog.DefaultExt := 'pas';
    SaveDialog.Filter := 'Pascal files (*.pas)|*.pas';
    SaveDialog.FilterIndex := 1;
    if SaveDialog.Execute then begin
      if FileExists(SaveDialog.FileName) then
        if MessageDlg('The file '+ExtractFileName(SaveDialog.FileName)+
             ' already exists.  Do you want to replace it?',
             mtConfirmation,[mbYes,mbNo],0) = mrNo then
          Exit;
        {fi}
        formDataMod.SaveCode;
        ShowMessage('Code written to '+SaveDialog.FileName);
      {fi}
      end
    else { Cancel SaveDialog }
      Exit;
    {fi}
    end
  else
    ShowMessage('No tables are open.');
  {fi}
end;

procedure TMainForm.mOpenDbClick(Sender: TObject);
var
  F: TSearchRec;
  Rslt: Integer;
  sTemp: string;
  aChild: TMDIChild;
  Cntr: Integer;
begin
  if AliasMode then
    with AliasForm do begin
      DlgMode := Alias;
      if ShowModal = mrOk then begin
        for Cntr := 0 to lbTables.Items.Count -1 do begin
          aChild := nil;
          aFileType := UnkType;
          if not CreateMDIChild(cbAlias.Text+'\'+lbTables.Items[Cntr],
                aFileType,aChild) and (aChild <> nil) then
            try
              aChild.Close;
            except

            end;
          {fi}
          end;
        {od for}
        end;
      {fi}
      end
    {od with}
  else begin
    if formOpenDb.ShowModal = mrOk then begin
      sTemp := formOpenDb.DbName;
      if Length(sTemp) > 3 then
        sTemp := sTemp + '\';
      {fi}
      Rslt := FindFirst(sTemp+'*.dbf',faAnyFile,F);
      if Rslt > 0 then
        ShowMessage('Folder does not contains dBASE tables.')
      else
        while Rslt =0 do begin
          aChild := nil;
          aFileType := UnkType;
          if not CreateMDIChild(sTemp + F.Name,aFileType,aChild) and
             (aChild <> nil) then
            try
              aChild.Close;
            except

            end;
          {fi}
          Rslt := FindNext(F);
          end;
        {od while}
      {fi}
      FindClose(F);
      end;
    {fi}
    end;
  {fi EditMode}
end;  { TMainForm.mOpendbClick }

procedure TMainForm.mDataModClick(Sender: TObject);
begin
  if MDIChildCount > 0 then begin
    formDataMod.LoadTables;
    formDataMod.ShowModal;
    end
  else
    ShowMessage('No tables are open.');
  {fi}
end;

procedure TMainForm.FileAlias1Execute(Sender: TObject);
begin
  AliasMode := not AliasMode;
  FileAlias1.Checked := AliasMode;
end;  { TMainForm.FileAlias1Execute }

procedure TMainForm.mDelAliasClick(Sender: TObject);
begin
  AliasForm.DlgMode := Delete;
  if AliasForm.ShowModal = mrOk then begin
    Session.DeleteAlias(AliasForm.cbAlias.Text);
    Session.SaveConfigFile;
    end;
  {fi}
end;

{***********************************************************************}
{                                                                       }
{  Checks to see if the table in the active window is also open in      }
{  another window.  Return True if there are no other instances.        }
{                                                                       }
{  Called by TMainForm.mEraseClick and TMainForm.mRenameClick           }
{                                                                       }
{***********************************************************************}
function TMainForm.NoDuplicate: Boolean;
var
  aPath: string;
  aTable: string;
  Cntr: Integer;
begin
  with ActiveMDIChild as TMDIChild do begin
    aPath := FilePath;
    aTable := Table1.TableName;
    end;
  {od with}
  Result := True;
  for Cntr := 1 to MDIChildCount - 1 do
    with MDIChildren[Cntr] as TMDIChild do
      if (Table1.TableName = aTable) and (FilePath = aPath) then begin
        Result := False;
        Break;
        end;
      {fi}
    {od with}
  {od for}
end;  { TMainForm.NoDuplicate }

procedure TMainForm.mEraseClick(Sender: TObject);
begin
  if NoDuplicate then begin
    with ActiveMDIChild as TMDIChild do begin
      if MessageDlg('Are you sure that you want to erase the table '+
          Table1.TableName+'?',mtConfirmation,[mbYes,mbNo],0) = mrYes then begin
        Table1.Active := False;
        try
          Table1.DeleteTable;
        except
          ShowMessage('This table is being used by another program '+
               'and cannot be erased.');
          Table1.Active := True;
          Exit;
        end{try};
        {fi}
        PostMessage(Self.Handle,CloseDb,0,0);
        end;
      {fi}
      end;
    {od with}
    end
  else
    ShowMessage('More than one copy of this table is open. '+
         'Close extra instance(s) and try again.');
  {fi}
end;

procedure TMainForm.mRenameClick(Sender: TObject);
var
  aValue: string;
begin
  if NoDuplicate then begin
    if InputQuery('Rename','Enter new name',aValue) then
      with ActiveMDIChild as TMDIChild do begin
        Table1.Active := False;
        try
          Table1.RenameTable(aValue);
        except
          ShowMessage('This table is being used by another program '+
               'and cannot be renamed.');
          Table1.Active;
          Exit;
        end;
        Table1.Active := True;
        CallName := GetCallName(ChangeFileExt(Table1.TableName,''));
        SetCaption;
        end;
      {od with}
    {fi}
    end
  else
    ShowMessage('More than one copy of this table is open. '+
         'Close extra instance(s) and try again.');
  {fi}
end;

end.
