{***********************************************************************}
{                                                                       }
{  New Form is used to:                                                 }
{                                                                       }
{    *  Create a new table                                              }
{    *  Clone an open table                                             }
{                                                                       }
{  Copyright (c) 2000, 2003 Douglas P. Baird - dpbaird@sprynet.com      }
{                                                                       }
{  Uses the following component not part of Delphi 5:                   }
{                                                                       }
{    * TDatagrid from DataGrid  EC Software 1997-1999 at               }
{      http://www.ec-software.com also available on Torry's Delphi      }
{      Pages at http://www.torry.webnorth.com/index.htm                 }
{                                                                       }
{***********************************************************************}

unit NewForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Mask, StdCtrls, Grids, Buttons, DataGrid, Main, dbvConst, ChildWin,
  ExtCtrls, Db, DBTables, Spin;

type
  TFormNew = class(TForm)
    DataGrid1: TDataGrid;
    GroupBox1: TGroupBox;
    btnUp: TBitBtn;
    btnDown: TBitBtn;
    btnCancel: TBitBtn;
    btnAdd: TBitBtn;
    btnRemove: TBitBtn;
    btnDefNdx: TBitBtn;
    RadioGroup1: TRadioGroup;
    btnSave: TBitBtn;
    btnAppend: TBitBtn;
    NewTable: TTable;
    Mblock: TSpinEdit;
    Label1: TLabel;
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnDefNdxClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnAppendClick(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure DataGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    LoadingLists: Boolean;
    procedure AnotherRow;
    function  CheckFieldName: Boolean;
    procedure SetFldType(var msg: TMessage); message SetType;
    function NumericCheck: Boolean; 
    procedure SetCell(var msg: TMessage); message SetColRow;
    function ShowCursor: string;
    function GetDbType: DbType;
    procedure SetDbType(aType: DbType);
  public
    { Public declarations }
    property TableType: DbType read GetDbType write SetDbType;
    procedure ClearLists; { Called by TMainForm.FileNew1Execute }
    procedure LoadLists(aChild: TMDIChild);  { Called by }
                                             { TMainForm.FileClone1Execute }
  end;  { TFormNew }

  {***********************************************************************}
  {                                                                       }
  {  The TDBaseFile class is used to access the dBASE file header when    }
  {  the floating point (F) field type is included in the record.  Delphi }
  {  does not support this field type directly, since it is redundant     }
  {  with the numeric (N) field type.  dBASEviewer creates the record     }
  {  using the numebric field type and then replaces the "N" in the file  }
  {  header with an "F".  The DBE honors the "F" when processing records. }
  {                                                                       }
  {***********************************************************************}

  TDBaseFile = class
    aFile: file of Byte;
    aBuffer: array of Byte;
    procedure FixDBase(aName: string; aGrid: TDataGrid);
    procedure FixMBlock(aName: string; aSize: Word);
  end;

var
  FormNew: TFormNew;
  aDBaseFile: TDBaseFile;

implementation

{$R *.DFM}

uses IndexGen, AliasDialog;

function TFormNew.ShowCursor: string;
begin
  with DataGrid1 do begin
    Result := Cells[0,Row];
    Cells[0,Row] := '**';
    SetFocus; 
    end;
  {od with}
end;  { TFormNew.ShowCursor }

function TFormNew.GetDbType: DbType;
begin
  case RadioGroup1.ItemIndex of
    0: Result := Lvl5Type;
    1: Result := Lvl7Type;
    2: Result := FoxProType;
  else
    Result := UnkType;
  end;
end;  { TFormNew.GetType }

procedure TFormNew.SetDbType(aType: DbType);
begin
  RadioGroup1.ItemIndex := Ord(aType) - 1;
end;  { TFormNew.SetType }

{***********************************************************************}
{  btnUpClick}
procedure TformNew.btnUpClick(Sender: TObject);
var
  sTemp: string;
  iTemp: Integer;
  Cntr: Integer;
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    if Row > 1 then begin
      iTemp := Row - 1;
      for Cntr := 0 to 4 do begin
        sTemp := Cells[Cntr,iTemp];
        Cells[Cntr,iTemp] := Cells[Cntr,Row];
        Cells[Cntr,Row] := sTemp;
        end;
      {od for}
      Row := iTemp;
      end;
    {fi}
    Cells[0,Row] := ShowCursor;
    SetFocus;
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od with}
end;  { TformNew.btnUpClick }

procedure TFormNew.btnDownClick(Sender: TObject);
var
  sTemp: string;
  iTemp: Integer;
  Cntr: Integer;
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    if Row < DataGrid1.RowCount then begin
      iTemp := Row + 1;
      if iTemp = RowCount then
        RowCount := iTemp + 1;
      for Cntr := 0 to ColCount-1 do begin
        sTemp := Cells[Cntr,iTemp];
        Cells[Cntr,iTemp] := Cells[Cntr,Row];
        Cells[Cntr,Row] := sTemp;
        end;
      {od for}
      Row := iTemp;
      end;
    {fi}
    SetFocus;
    Cells[0,Row] := ShowCursor;
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od with}
end;  { TFormNew.btnDownClick }

procedure TFormNew.AnotherRow;
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    if Length(Cells[0,RowCount-1]) > 0 then begin
      RowCount := RowCount + 1;
      Row := Row + 1;
      Col := 0;
      end
    else begin
      Row := RowCount-1;
      Col := 0;
      end;
    {fi}
    PostMessage(Self.Handle,SetColRow,0,Row);
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od with}
end;  { TFormNew.AnotherRow }

function  TFormNew.CheckFieldName: Boolean;
const
  ValidChar = ['a'..'z','A'..'Z','0'..'9','_'];
var
  sTemp: String;
  Cntr: Integer;
begin
  Result := True;
  with DataGrid1 do begin
    sTemp := Cells[0,Row];
    for Cntr := 1 to Length(sTemp) do
      if not (sTemp[Cntr] in ValidChar) then
        if not ((sTemp[Cntr] = ' ') and
                (TableType = Lvl7Type)) then begin
          ShowMessage('The character ''' + sTemp[Cntr] +
               ''' is not valid in a field name');
          Result := False;
          Exit;
          end;
        {fi}
      {fi}
    {od for}
    end;
  {od with}
end;  { TFormNew.CheckFieldName }

procedure TFormNew.SetFldType(var msg: TMessage);
type
  ColMask = set of (cmSize,cmDec);
var
  sTemp: string;
{func/proc}
  procedure SetCols(aType,aSize,aDec: string; aMask: ColMask);
  begin
    with DataGrid1 do begin
      OnSelectCell := nil;
      Cells[2,Msg.LParamLo] := aType;
      if cmSize in aMask then Cells[3,Msg.LParamLo] := aSize;
      if cmDec in AMask then Cells[4,Msg.LParamLo] := aDec;
      if not (cmSize in aMask) then begin
        Col := 3;
        Row := Msg.LParamLo;
        end
      else begin
        if (TableType = Lvl7Type) and (aType <> '+') then begin
          Row := Msg.LParamLo;
          Col := 5;
          end
        else
          AnotherRow;
        {fi}
        end;
      {fi}
      OnSelectCell := DataGrid1SelectCell;
      end;
    {od with}
  end;
begin  { TFormNew.SetFldType }
  with DataGrid1 do begin
    sTemp := Cells[1,Msg.LParamLo];
    if sTemp = 'Autoincrement' then SetCols('+','4','',[cmSize,cmDec])
    else if sTemp = 'Binary' then SetCols('B','10','',[cmSize,cmDec])
    else if sTemp = 'Character' then SetCols('C','','',[cmDec])
    else if sTemp = 'Date' then SetCols('D','8','',[cmSize,cmDec])
    else if sTemp = 'Float' then SetCols('F','','',[])
    else if sTemp = 'Integer' then SetCols('I','4','',[cmSize,cmDec])
    else if sTemp = 'Logical' then SetCols('L','1','',[cmSize,cmDec])
    else if sTemp = 'Memo' then SetCols('M','10','',[cmSize,cmDec])
    else if sTemp = 'Number' then SetCols('N','','',[])
    else if sTemp = 'OLE' then SetCols('G','10','',[cmSize,cmDec])
    else if sTemp = 'Real64' then SetCols('O','8','',[cmSize,cmDec])
    else if sTemp = 'Timestamp' then SetCols('@','8','',[cmSize,cmDec]);
    {fi}
    end;
  {od with}
end;  { TFormNew.SetFldType }

procedure TFormNew.SetCell(var Msg: TMessage);
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    Col := Msg.WParamLo;
    Row := Msg.LParamLo;
    SetFocus;
    Cells[0,Row] := ShowCursor;
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od with}
end; { TFormNew.SetCell }

 function TFormNew.NumericCheck: Boolean;
var
  sTemp: string;
{func/proc}
  procedure NoDec(aSize: Integer);
  begin
    with DataGrid1 do begin
      if (Col = 3) and (IntValue[3,Row] <> 4) then begin
        ShowMessage('Size for '+sTemp+' must be '+ IntToStr(aSize));
        Cells[3,Row] := IntToStr(aSize);
        end
      else if (Col = 4) and (Length(Cells[4,Row]) > 0) then begin
        ShowMessage('Dec for '+sTemp+' must be blank');
        Result := False;
        PostMessage(Self.Handle,SetColRow,4,Row);
        end;
      {fi}
      end;
    {od with}
  end;  { NoDec}
  procedure NeedBoth;
  var
    DecLen: Integer;
  begin
    with DataGrid1 do begin
      if (Col = 3) and (IntValue[3,Row] = 0) then begin
        ShowMessage('Size for '+sTemp+' must be specified');
        Result := False;
        PostMessage(Self.Handle,SetColRow,3,Row);
        end
      else if Col = 4 then begin
        try
          DecLen := StrToInt(Cells[4,Row]);
          if DecLen > StrToInt(Cells[3,Row])-2 then begin
            ShowMessage('Decimals for '+sTemp+' must be at least two less '+
                'than the field size');
            Result := False;
            PostMessage(Self.Handle,SetColRow,4,Row);
            end
          else if TableType = Lvl7Type then
            PostMessage(Self.Handle,SetColRow,5,Row)
          else
            AnotherRow;
          {fi}
        except
          on EConvertError do begin
            ShowMessage('Decimals for '+sTemp+' must be specified');
            PostMessage(Self.Handle,SetColRow,4,Row);
            end;
          {od on}
        end; {try}
        end;
      {fi}
      end;
    {od with}
  end;  {NeedBoth}
begin  { TFormNew.NumericCheck }
  Result := True;
  with DataGrid1 do begin
    sTemp := Cells[1,Row];
    if sTemp = 'Autoincrement' then
      NoDec(4)
    else if sTemp = 'Binary' then
      NoDec(10)
    else if sTemp = 'Character' then
      with DataGrid1 do begin
        if Col = 3 then
          if IntValue[3,Row] = 0 then begin
            ShowMessage('Size for '+sTemp+' must be specified');
            Result := False;
            PostMessage(Self.Handle,SetColRow,3,Row);
            end
          else if TableType = Lvl7Type then
            PostMessage(Self.Handle,SetColRow,5,Row)
          else
            AnotherRow
          {fi}
        else if (Col = 4) and (Length(Cells[4,Row]) > 0) then begin
          ShowMessage('Dec for '+sTemp+' must not be specified');
          Cells[4,Row] := '';
          end;
        {fi}
        end
      {od with}
    else if sTemp = 'Date' then
      NoDec(8)
    else if sTemp = 'Float' then
      NeedBoth
    else if sTemp = 'Integer' then
      NoDec(4)
    else if sTemp = 'Logical' then
      NoDec(1)
    else if sTemp = 'Memo' then
      NoDec(10)
   else if sTemp = 'Number' then
     NeedBoth
    else if sTemp = 'OLE' then
      NoDec(10)
    else if sTemp = 'Real64' then
      NoDec(8)
    else if sTemp = 'Timestamp' then
      NoDec(8)
    else begin
      ShowMessage(sTemp + ' is not a valid Field Type');
      Result := False;
      PostMessage(Self.Handle,SetColRow,1,Row);
      end;
    {fi}
    end;
  {od with}
end;  { TFormNew.NumericCheck }

procedure TFormNew.btnAppendClick(Sender: TObject);
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    if Length(Cells[0,RowCount-1]) > 0 then
      RowCount := RowCount + 1;
    {fi}
    Col := 0;
    Row := RowCount - 1;
    SetFocus;
    Cells[0,Row] := ShowCursor;
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od with}
end;

procedure TFormNew.btnAddClick(Sender: TObject);
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    InsertRow(Row);
    SetFocus;
    Cells[0,Row] := ShowCursor;
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od with}
end;  { TFormNew.btnAddClick }

procedure TFormNew.btnRemoveClick(Sender: TObject);
var
  sTemp: string;
begin
  with DataGrid1 do begin
    OnSelectCell := nil;
    sTemp := Cells[0,Row];
    if Length(sTemp) = 0 then
      sTemp := 'the field';
    {fi}
    if MessageDlg('Are you shure that you want to remove ' +
        sTemp + '?', mtConfirmation,[mbYes,mbNo],0) = mrYes then
      DeleteRow(Row);
    {fi}
    Col := 0;
    SetFocus;
    Cells[0,Row] := ShowCursor;
    OnSelectCell := DataGrid1SelectCell;
    end;
  {od for}
end;  { TFormNew.btnRemoveClick }

procedure TFormNew.btnCancelClick(Sender: TObject);
begin
  if MessageDlg('Are you sure that you want to cancel?',
       mtConfirmation,[mbYes,mbNo],0) = mrYes then
     ModalResult := mrCancel
  else
     ModalResult := mrNone;
  {fi}
end;  { TFormNew.btnCancelClick }

procedure TDBaseFile.FixDBase(aName: string; aGrid: TDataGrid);
var
  aSize: Integer;
  Cntr: Integer;
{func/proc}
  procedure PatchFile(aField: Integer; aChar: Char);
  begin
    if FormNew.TableType = Lvl7Type then
      aBuffer[52+48*aField] := byte(aChar)
    else
      aBuffer[11+32*aField] := byte(aChar);
    {fi}
  end;
begin
  AssignFile(aFile,aName);
  Reset(aFile);
  aSize := FileSize(aFile);
  SetLength(aBuffer,aSize);
  BlockRead(aFile,Pointer(Pointer(aBuffer)^),aSize);
  for Cntr := 0 to aGrid.RowCount - 1 do begin
      if aGrid.Cells[1,Cntr] = 'Float' then
        PatchFile(Cntr,'F')
      {fi}
    end;
  {od for}
  Rewrite(aFile);
  BlockWrite(aFile,Pointer(Pointer(aBuffer)^),aSize);
  CloseFile(aFile);
  SetLength(aBuffer,0);
end;  { TDBaseFile.FixDBase }

procedure TDBaseFile.FixMBlock(aName: string; aSize: Word);
{***********************************************************************}
{  Checks to see if a memo field file for the dBASE file aName exists   }
{  and if it does fix the the block size to aSize and rewrite the file. }
{***********************************************************************}
begin
  if (FormNew.TableType = FoxProType) then begin
    aName := ChangeFileExt(aName,'.FPT');
    if not FileExists(aName) then
      aName := '';
    {fi}
    end
  else begin
    aName := ChangeFileExt(aName,'.DBT');
    if not FileExists(aName) then
      aName := '';
    {fi}
    end;
  {fi}
  if Length(aName) > 0 then begin
    AssignFile(aFile,aName);
    Reset(aFile);
    SetLength(aBuffer,1024);
    BlockRead(aFile,Pointer(Pointer(aBuffer)^),1024);
    if FormNew.TableType = FoxProType then begin
      aBuffer[6] := Hi(aSize);
      aBuffer[7] := Lo(aSize);
      end
    else begin
      aBuffer[20] := Lo(aSize);
      aBuffer[21] := Hi(aSize);
      end;
    {fi}
    Rewrite(aFile);
    BlockWrite(aFile,Pointer(Pointer(aBuffer)^),aSize);
    CloseFile(aFile);
    SetLength(aBuffer,0);
    end;
  {fi}
end;

procedure TFormNew.btnDefNdxClick(Sender: TObject);
begin
  formCreateNdx.Setup;
  formCreateNdx.ShowModal;
end;  { TFormNew.btnDefNdxClick }

procedure TFormNew.ClearLists;
var
  Cntr: Integer;
begin
  with DataGrid1 do begin
    for Cntr := RowCount-1 downto 1 do
      Rows[Cntr].Clear;
    {od for}
    RowCount := 2;
    end;
  {od with}
  with formCreateNDX.FieldGrid do begin
    for Cntr := RowCount-1 downto 1 do
      Rows[Cntr].Clear;
    {od for}
    RowCount := 2
    end;
  {od with}
  with formCreateNDX.IndexGrid do begin
    for Cntr := RowCount-1 downto 1 do
      Rows[Cntr].Clear;
    {od for}
    RowCount := 2;
    end;
  {od with}
  with formCreateNDX do begin
    cbExpression.Checked := False;
    cbDescending.Checked := False;
    cbUnique.Checked := False;
    cbPrimary.Checked := False;
    end;
  {od with}
end;  { TFormNew.ClearLists }

procedure TFormNew.LoadLists(aChild: TMDIChild);
var
  DefsCntr: Integer;
  GridCntr: Integer;
  FldType: string;
  sTemp: string;
{func/proc}
  procedure TestOptions(OptStr: string);
  begin
    case TableType of
      Lvl5Type: begin
        if (Pos('U',OptStr) <> 0) or (Pos('P',OptStr) <> 0) then begin
          TableType := Lvl7Type;
          ShowMessage('Unsupported index option found.  '+
              'Changing dBASE 5 to dBASE 7.');
          end;
        {fi}
        end;
      FoxProType: begin
        if (Pos('P',OptStr) <> 0) then begin
          TableType := Lvl7Type;
          ShowMessage('Unsupported index option found.  '+
              'Changing FoxPro to dBASE 7.');
          end;
        {fi}
        end;
    end; {case}
  end;  { TestOptions }
begin
  DataGrid1.OnSelectCell := nil;
  LoadingLists := True;
  ClearLists;
  with aChild do begin
    TableType := dBASEtype;
    if MemoBlock = 0 then
      Mblock.Text := '1024'
    else
      Mblock.Text := IntToStr(MemoBlock);
    {fi}
    DataGrid1.RowCount := Table1.FieldDefs.Count + 1;
    for DefsCntr := 0 to Table1.FieldDefs.Count - 1 do begin
      GridCntr := DefsCntr + 1;
      DataGrid1.Cells[0,GridCntr] := TrimRight(TableStructure[DefsCntr].Name);
      DataGrid1.Cells[2,GridCntr] := TableStructure[DefsCntr].FieldType;
      DataGrid1.Cells[3,GridCntr] := TableStructure[DefsCntr].Length;
      DataGrid1.Cells[4,GridCntr] := TableStructure[DefsCntr].Decimal;
      DataGrid1.Cells[5,GridCntr] := TableStructure[DefsCntr].Required;
      FldType := DataGrid1.Cells[2,GridCntr];
      if FldType = '+' then sTemp := 'Autoincrement'
      else if FldType = 'B' then sTemp := 'Binary'
      else if FldType = 'C' then sTemp := 'Character'
      else if FldType = 'N' then sTemp := 'Number'
      else if FldType = 'D' then sTemp := 'Date'
      else if FldType = 'F' then sTemp := 'Float'
      else if FldType = 'I' then sTemp := 'Integer'
      else if FldType = 'L' then sTemp := 'Logical'
      else if FldType = 'M' then sTemp := 'Memo'
      else if FldType = 'G' then sTemp := 'OLE'
      else if FldType = 'O' then sTemp := 'Real64'
      else if FldType = '@' then sTemp := 'Timestamp'
      else sTemp := DataGrid1.Cells[2,GridCntr] + ' Unknown';
      {fi}
      DataGrid1.Cells[1,GridCntr] := sTemp;
      end;
    {od for}
    Table1.IndexDefs.Update;
    if Table1.IndexDefs.Count > 0 then begin
      formCreateNdx.IndexGrid.RowCount := Table1.IndexDefs.Count+1;
      for DefsCntr := 0 to Table1.IndexDefs.Count-1 do begin
        GridCntr := DefsCntr + 1;
        formCreateNdx.IndexGrid.Cells[0,GridCntr] :=
             Table1.IndexDefs[DefsCntr].Name;
        if ixExpression in Table1.IndexDefs[DefsCntr].Options then
          formCreateNdx.IndexGrid.Cells[1,GridCntr] :=
               Table1.IndexDefs[DefsCntr].Expression
        else
          formCreateNdx.IndexGrid.Cells[1,GridCntr] :=
               Table1.IndexDefs[DefsCntr].Fields;
        {fi}
        formCreateNdx.IndexGrid.Cells[2,GridCntr] :=
               GetOptions(Table1.IndexDefs[DefsCntr].Options);
        TestOptions(formCreateNdx.IndexGrid.Cells[2,GridCntr]);
        end;
      {od for}
      end
    else begin
      with formCreateNDX.IndexGrid do begin
        for GridCntr := RowCount-1 downto 1 do
          Rows[GridCntr].Clear;
        {od for}
        RowCount := 2;
        end;
      {od with}
      end;
    {fi}
    end;
  {od with}
  LoadingLists := False;
  DataGrid1.OnSelectCell := DataGrid1SelectCell;
end;  { TFormNew.LoadLists }

procedure TFormNew.RadioGroup1Click(Sender: TObject);
const
  dBASE7pick = 'Autoincrement,Binary,Character,' +
      'Date,Float,Integer,Logical,' +
      'Memo,Number,OLE,Real64,Timestamp';
  dBASE5pick = 'Binary,Character,Date,Float,' +
      'Logical,Memo,Number,OLE';
  FoxPropick = 'Character,Date,Float,Logical,' +
      'Memo,Number,OLE';
{func/proc}
  procedure FixFields;
  var
    NameChanged: Boolean;
    TypeChanged: Boolean;
    Cntr: Integer;
    aLen: Integer;
    sTemp: string;
  begin
    NameChanged := False;
    TypeChanged := False;
    for Cntr := 1 to DataGrid1.RowCount-1 do
      with DataGrid1 do begin
        aLen := Length(Cells[0,Cntr]);
        if aLen > 0 then begin
          if aLen > Columns[0].MaxLength then begin
            Cells[0,Cntr] := Copy(Cells[0,Cntr],1,Columns[0].MaxLength);
            NameChanged := True;
            end;
          {fi}
          if Columns[1].PickList.IndexOf(Cells[1,Cntr]) = -1 then begin
            Cells[1,Cntr] := '';
            TypeChanged := True;
            end;
          {fi}
          end;
        {fi}
        end;
      {od with}
    {od for}
    sTemp := '';
    if NameChanged then
      sTemp := 'name(s)';
    {fi}
    if TypeChanged then
      if Length(sTemp) > 0 then
        sTemp := sTemp + ' and type(s)'
      else
        sTemp := 'type(s)';
    {fi}
    if Length(sTemp) > 0 then
      MessageDlg('Field definition(s) changed.  Check field '+sTemp+'.',
        mtWarning,[mbOk],0);
    {fi}
  end;
begin
  case TableType of
    Lvl5Type: begin
         DataGrid1.Columns[0].MaxLength := 10;
         DataGrid1.Columns[1].PickList.CommaText := dBASE5pick;
         DataGrid1.ColCount := 5;
         end;
    Lvl7Type: begin
         DataGrid1.Columns[0].MaxLength := 32;
         DataGrid1.Columns[1].PickList.CommaText := dBASE7pick;
         DataGrid1.ColCount := 6;
         DataGrid1.ColWidths[5] := 37;
         DataGrid1.Cells[5,0] := 'Req''d';
         DataGrid1.Columns[5].PickList.CommaText := '"Y","N"';
         end;
    FoxProType: begin
         DataGrid1.Columns[0].MaxLength := 10;
         DataGrid1.Columns[1].PickList.CommaText := FoxPropick;
         DataGrid1.ColCount := 5;
         end;
  end;
  FixFields;
  if not LoadingLists then
    PostMessage(Self.Handle,SetColRow,0,DataGrid1.Row);
  {fi}
end;  { TFormNew.RadioGroup1Click }

procedure TFormNew.btnSaveClick(Sender: TObject);
var
  MessageResult: word;
  DialogResult: TModalResult;
  FloatPatch: Boolean;
  aType: string;
  aRes: Boolean;
  sTemp: string;
  NdxOptions: TIndexOptions;
  Continue: Boolean;
  aFileNm: string;
{func/proc}
  procedure DeleteOthers(aName: string);
  const
    ExtList: array[0..3] of string[4] = ('.MDX','.DBT','.CDX','.FPT');
  var
    Cntr: Integer;
    sTemp: string;
  begin
    for Cntr := 0 to 4 do begin
      sTemp := ChangeFileExt(aName,ExtList[Cntr]);
      if FileExists(sTemp) then
        DeleteFile(sTemp);
      {fi}
      end;
    {od for}
  end;  { DeleteOthers }
  function DefineFields: Boolean;
  var
    Cntr: Integer;
  begin
    Result := True;
    with NewTable do
      with FieldDefs do begin
        Clear;
        for Cntr := 1 to DataGrid1.RowCount - 1 do begin
          sTemp := DataGrid1.Cells[0,Cntr];
          aType := DataGrid1.Cells[1,Cntr];
          if FormNew.TableType = Lvl7Type then
            aRes := (DataGrid1.Cells[5,Cntr] = 'Y')
          else
            aRes := False;
          {fi}
          if Length(sTemp) > 0 then begin
            if aType = 'Autoincrement' {'+'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType := ftAutoInc;
                end
              {od with}
            else if aType = 'Binary' {'B'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType := ftTypedBinary;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Character' {'C'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType :=ftString;
                Size := StrToInt(FormNew.DataGrid1.Cells[3,Cntr]);
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Date' {'D'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType := ftDate;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Float' {'F'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType := ftBcd;
                Precision :=StrToInt(FormNew.DataGrid1.Cells[3,Cntr]);
                Size := StrToInt(FormNew.DataGrid1.Cells[4,Cntr]);
                if aRes then
                  Required := True;
                {fi}
                FloatPatch := True;
                end
              {od with}
            else if aType = 'Integer' {'I'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType := ftInteger;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Logical' {'L'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType :=ftBoolean;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Memo' {'M'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType :=ftMemo;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Number' {'N'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType := ftBcd;
                Precision := StrToInt(FormNew.DataGrid1.Cells[3,Cntr]);
                Size := StrToInt(FormNew.DataGrid1.Cells[4,Cntr]);
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'OLE' {'G'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType :=ftDBaseOle;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Real64' {'O'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType :=ftFloat;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else if aType = 'Timestamp' {'@'} then
              with AddFieldDef do begin
                Name := sTemp;
                DataType :=ftDateTime;
                if aRes then
                  Required := True;
                {fi}
                end
              {od with}
            else begin
              ShowMessage('No field type defined for "'+sTemp+'"');
              Result := False;
              end
            end;
          {fi}
          end;
        {od for Cntr}
        end;
      {od with FieldDefs}
    {od with NewTable}
  end;  { DefineFields }
  procedure DefineIndexes;
  var
    Cntr: Integer;
  begin
    with NewTable do
      with formCreateNdx.IndexGrid do
        with IndexDefs do begin
          Clear;
          for Cntr := 1 to RowCount - 1 do
            if Length(Cells[0,Cntr]) > 0 then begin
              NdxOptions := [];
              if Pos('E',Cells[2,Cntr]) > 0 then
                NdxOptions := NdxOptions + [ixExpression];
              {fi}
              if Pos('D',Cells[2,Cntr]) > 0 then
                NdxOptions := NdxOptions + [ixDescending];
              {fi}
              if Pos('U',Cells[2,Cntr]) > 0 then
                NdxOptions := NdxOptions + [ixUnique];
              {fi}
              if Pos('P',Cells[2,Cntr]) > 0 then
                NdxOptions := NdxOptions + [ixPrimary];
              {fi}
              if Pos('E',Cells[2,Cntr]) = 0 then
                with AddIndexDef do begin
                  Name :=Cells[0,Cntr];
                  Fields := Cells[1,Cntr];
                  Options := NdxOptions;
                  end
                {od with}
              else
                with AddIndexDef do begin
                  Name :=Cells[0,Cntr];
                  Expression := Cells[1,Cntr];
                  Options := NdxOptions;
                  end;
                {od with}
              {fi}
              {od with ..}
              end;
            {fi}
          {od for}
          end;
        {od withIndexDefs}
      {od with IndexGrid}
    {od with NewTable}
  end;
begin  { TFormNew.btnSaveClick }
  Continue := False;
  DialogResult := mrNone;
  if MainForm.AliasMode then begin
    AliasForm.DlgMode := Save;
    if AliasForm.ShowModal = mrOk then begin
      Continue := True;
      aFileNm := MainForm.AliasToFile(AliasForm.cbAlias.Text+'\'+
                   AliasForm.edFile.Text);
      MainForm.aFileName := AliasForm.cbAlias.Text+'\'+AliasForm.edFile.Text;
      end;
    {fi}
    end
  else
    with MainForm do begin
      SaveDialog.Title := 'Save New Table';
      SaveDialog.InitialDir := OpenDialog.InitialDir;
      SaveDialog.FileName := '';
      SaveDialog.DefaultExt := 'dbf';
      SaveDialog.Filter := 'dBase Files (.dbf)|*.DBF';
      SaveDialog.FilterIndex := 1;
      if SaveDialog.Execute then begin
        Continue := True;
        aFileNm := SaveDialog.FileName;
        MainForm.aFileName := aFileNm;
        end;
      {fi}
      end;
    {od with}
  {fi}
  with MainForm do begin
    if Continue then begin
      if FileExists(aFileNm) then begin
        MessageResult := MessageDlg('The table '+ aFileName +
           ' already exits.  Do you want to overwrite it?',
           mtConfirmation,[mbYes, mbNo],0);
        case MessageResult of
          mrYes: begin
            if DeleteFile(aFileNm)then begin
              DeleteOthers(aFileNm);
              DialogResult := mrOk;
              end
            else begin
              ShowMessage('The file '+aFileName+
                 ' is busy.  Select another file name.');
              DialogResult := mrNone;
              end;
            {fi}
            end;
          mrNo: DialogResult := mrNone;  { Try another filename }
        end {case};
        end {FileExists}
      else {does not exist} begin
        DialogResult := mrOk;
        end;
      {fi FileExists}
      end
    else                   
      DialogResult := mrNone;
    {fi Continue}
    { Save Execute}
    if DialogResult = mrOk then begin
      FloatPatch := False;
      with NewTable do begin
        if AliasMode then
          DatabaseName := AliasForm.cbAlias.Text
        else
          DatabaseName := ExtractFileDir(aFileNm);
        {fi}
        TableName := ExtractFileName(aFileNm);
        case FormNew.TableType of
          Lvl5Type: begin
              TableType := ttDBase;
              TableLevel := 5;
            end;
          Lvl7Type: begin
              TableType := ttDBase;
              TableLevel := 7;
            end;
          FoxProType: begin
              TableType := ttFoxPro;
              TableLevel := 0;
            end;
        end; {case}
        if DefineFields then begin
          DefineIndexes;
          try
            CreateTable;
          except
            on E: Exception do begin
              MessageDlg('Error creating table. '+E.Message,mtError,[mbOk],0);
              Exit;
              end;
            {od on}
          end; {Try}
          end
        else
          DialogResult := mrNone;
        {fi}
        end;
      {od with NewTable}
      if DialogResult = mrOk then begin
        if FloatPatch then
          aDBaseFile.FixDBase(aFileNm,DataGrid1);
        {fi}
        if Mblock.Value <> 1024 then  { change default block size }
          aDBaseFile.FixMBlock(aFileNm,StrToInt(Mblock.Text));
        {fi}
        NewTable.Active := True;
        SetDbActions(True,False);
        end;
      {fi mrOk}
      end  { DialogResults Ok }
    else
      DialogResult := mrNone;
    {fi DialogResult}
    NewTable.Active := False;
    end;
  {od with MainForm}
  ModalResult := DialogResult;
end;  { TFormNew.btnSaveClick }

procedure TFormNew.DataGrid1SelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  CanSelect := True;
  if Length(DataGrid1.Cells[0,DataGrid1.Row]) > 0 then
    with DataGrid1 do begin
      if (Col = 0) then
        CanSelect := CheckFieldName
      else if (Col = 1) then
        PostMessage(Self.Handle,SetType,2,Row)
      else if (Col = 3) then
        CanSelect := NumericCheck
      else if (Col = 4) then
        CanSelect := NumericCheck;
      {fi}
      end
    {od with}
  else if (ARow = DataGrid1.Row) then begin
    ShowMessage('A field name must be specified.');
    CanSelect := False;
    end;
  {fi}
end;

procedure TFormNew.FormActivate(Sender: TObject);
begin
  with DataGrid1 do begin
    Cells[0,1] := ShowCursor;
    SetFocus;
    end;
  {of with}
end;

initialization
  aDBaseFile := TDBaseFile.Create;

finalization
  aDBaseFile.Free;

end.

