unit PrintPv2;
{
	This unit is the progress bar form that can be shown during printing.
  When the   ShowProgressBar value is set to true, this form is displayed
  during the initial printing instructions that get stored, and then during
  the actual printing sequence to the printer. There are two ways that the
  progress bar can be changed:

    1. UpdateBar: This allows you to set the value of the progress bar directly.
    2. IncBar: This allows you to increment the value of the progress bar by
       the value passed.

  The TPrintObject component provides three ways to increment the progress bar.
  The first two as outlined above, and a third that sends the automatic
  increment value to the IncBar procedure.

  Added by Doug Baird  9/23/1999

  Two published Boolean properties are added to TPrintObject: Running and
  ShowCancel.  They access RunningFlag and ShowCancelFlag in this unit to
  store the actual values.  The values can be set at design time.

  The Cancel button is optional, controlled by ShowCancelFlag and set by
  OnShow.  The variable RunningFlag is set false when the Cancel button is
  clicked.  The value of RunningFlag is accesible through the publish property
  Running in TPrintObject and can be tested by the print routine to cancel
  printing.
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Gauges, StdCtrls, ExtCtrls, ComCtrls, Buttons;

type
  TPrintBarForm = class(TForm)
    Panel1: TPanel;
    PrintCaption: TLabel;
    Panel2: TPanel;
    PrintBar: TProgressBar;
    btnCancel: TBitBtn;
    procedure btnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    { The difference between UpdateBar and IncBar is that UpdateBar will
      set the progress bar to a specific value and IncBar will add the value
      to the current progress value. Either can be used by the program.
    }
    procedure UpdateBar(x: Integer);
    procedure IncBar(x: Integer);
    procedure SetDisplayText(s: string);
  end;

var
  PrintBarForm: TPrintBarForm;
  ShowCancelFlag: Boolean;
  RunningFlag: Boolean;

implementation

{$R *.DFM}

procedure TPrintBarForm.UpdateBar(x: Integer);
begin
  PrintBar.Position := x;
  Update;
  Application.ProcessMessages;
end;

procedure TPrintBarForm.IncBar(x: Integer);
begin
  with PrintBar do
    if (Position + x) > Max then
      Position := Max
    else
      Position := Position + x;
    {fi}
  {od with}
  Update;
  Application.ProcessMessages;
end;

procedure TPrintBarForm.SetDisplayText(s: string);
begin
  PrintCaption.Caption := s;
  Update;
  Application.ProcessMessages;
end;

procedure TPrintBarForm.btnCancelClick(Sender: TObject);
begin
  RunningFlag := False;
  btnCancel.Enabled := False
end;

procedure TPrintBarForm.FormShow(Sender: TObject);
begin
  if ShowCancelFlag then begin
    btnCancel.Visible := True;
    btnCancel.Enabled := True;
    Height := 142;
    end
  else begin
    btnCancel.Visible := False;
    Height := 100;
    end;
  {fi}
end;

end.
